@@ -1,13 +1,10 @@
-Tabmix._eval = function(name, code) name ? eval(name + " = " + code) : eval("(" + code + ")"); // jshint ignore:line
-// Tabmix._eval is on the first line to help us get the right line number
-
-/* jshint strict: false */
+/* eslint strict: 0 */
 
 // don't use strict for this file
-// so we don't evaluat all code as strict mode code
+// so we don't evaluate all code as strict mode code
 
 // aOptions can be: getter, setter or forceUpdate
-Tabmix.changeCode = function(aParent, aName, aOptions) {
+Tabmix.changeCode = function(aParent, afnName, aOptions) {
   let console = TabmixSvc.console;
   let debugMode = this._debugMode;
 
@@ -23,17 +20,28 @@ Tabmix.changeCode = function(aParent, aName, aOptions) {
     if (options && (options.setter || options.getter)) {
       this.type = options.setter ? "__lookupSetter__" : "__lookupGetter__";
       this.value = this.obj[this.type](this.fnName).toString();
-    }
-    else if (typeof this.obj[this.fnName] == "function")
+    } else if (typeof this.obj[this.fnName] == "function") {
       this.value = this.obj[this.fnName].toString();
-    else
+    } else {
       this.errMsg = "\n" + this.fullName + " is undefined.";
+    }
     this.notFound = [];
   }
 
   ChangeCode.prototype = {
     value: "", errMsg: "",
-    _replace: function TMP_utils__replace(substr ,newString, aParams) {
+    _replace: function TMP_utils__replace(substr, newString, aParams) {
+      // Don't insert new code before "use strict";
+      if (substr == "{") {
+        let re = /['|"]use strict['|"];/;
+        let result = re.exec(this.value);
+        if (result) {
+          if (!newString.startsWith("$&")) {
+            newString = newString.replace(substr, result[0] + "\n");
+          }
+          substr = result[0];
+        }
+      }
       var silent;
       if (typeof aParams != "undefined") {
         let doReplace, flags;
@@ -41,8 +49,7 @@ Tabmix.changeCode = function(aParent, aName, aOptions) {
           doReplace = "check" in aParams ? aParams.check : true;
           flags = aParams.flags;
           silent = aParams.silent;
-        }
-        else if (typeof aParams == "boolean") {
+        } else if (typeof aParams == "boolean") {
           doReplace = aParams;
         }
         if (!doReplace)
@@ -51,13 +58,13 @@ Tabmix.changeCode = function(aParent, aName, aOptions) {
           substr = new RegExp(substr.replace(/[{[(\\^.$|?*+\/)\]}]/g, "\\$&"), flags);
       }
 
-      var exist = typeof(substr) == "string" ? this.value.indexOf(substr) > -1 : substr.test(this.value);
+      var exist = typeof (substr) == "string" ? this.value.indexOf(substr) > -1 : substr.test(this.value);
       if (exist) {
         this.value = this.value.replace(substr, newString);
         this.needUpdate = true;
-      }
-      else if (!silent)
+      } else if (!silent) {
         this.notFound.push(substr);
+      }
       return this;
     },
 
@@ -78,38 +85,45 @@ Tabmix.changeCode = function(aParent, aName, aOptions) {
             ' }';
         }
         let [obj, fnName] = [aObj || this.obj, aName || this.fnName];
-        if (this.isValidToChange(fnName)) {
+        if (this.isValidToChange(this.fullName)) {
           if (obj)
-            Tabmix.setNewFunction(obj, fnName, Tabmix._eval(null, this.value));
+            Tabmix.setNewFunction(obj, fnName, Tabmix._makeCode(null, this.value));
           else
-            Tabmix._eval(fnName, this.value);
+            Tabmix._makeCode(fnName, this.value);
         }
         if (aShow)
           this.show(obj, fnName);
       } catch (ex) {
-        Components.utils.reportError("Tabmix " + console.callerName() +
-                                     " failed to change " + this.fullName +
-                                     "\nError: " + ex.message);
+        console.reportError(ex, console.callerName() + " failed to change " +
+                            this.fullName + "\nError: ");
       }
     },
 
     defineProperty: function(aObj, aName, aCode) {
       if (!this.type)
-        throw "Tabmix:\n" +  this.fullName + " don't have setter or getter";
+        throw new Error("Tabmix:\n" + this.fullName + " don't have setter or getter");
 
       let [obj, fnName] = [aObj || this.obj, aName || this.fnName];
       let descriptor = {enumerable: true, configurable: true};
 
+      let removeSpaces = function(match, p1, p2) {
+        return p1 + p2.replace(/\s/g, '_');
+      };
+
       let setDescriptor = function(type) {
         let fnType = "__lookup#ter__".replace("#", type);
         type = type.toLowerCase();
         let code = aCode && aCode[type + "ter"] ||
                    this.type == fnType ? this.value : obj[fnType](fnName);
 
-        if (typeof code == "string")
-          descriptor[type] = Tabmix._eval(null, code);
-        else if (typeof code != "undefined")
+        if (typeof code == "string") {
+          // bug 1255925 - Give a name to getters/setters add space before the function name
+          // replace function get Foo() to function get_Foo()
+          code = code.replace(/(^function\s*)(.*\()/, removeSpaces);
+          descriptor[type] = Tabmix._makeCode(null, code);
+        } else if (typeof code != "undefined") {
           descriptor[type] = code;
+        }
       }.bind(this);
 
       setDescriptor("Get");
@@ -136,23 +150,23 @@ Tabmix.changeCode = function(aParent, aName, aOptions) {
         let str = (notFoundCount > 1 ? "s" : "") + "\n    ";
         console.clog(caller + " was unable to change " + aName + "." +
           (this.errMsg || "\ncan't find string" + str + this.notFound.join("\n    ")) +
-          "\n\nTry Tabmix latest development version from tmp.garyr.net/tab_mix_plus-dev-build.xpi," +
-          "\nReport about this to Tabmix developer at http://tmp.garyr.net/forum/");
+          "\n\nTry Tabmix latest development version from tabmixplus.org/tab_mix_plus-dev-build.xpi," +
+          "\nReport about this to Tabmix developer at http://tabmixplus.org/forum/");
         if (debugMode)
           console.clog(caller + "\nfunction " + aName + " = " + this.value);
-      }
-      else if (!this.needUpdate && debugMode)
+      } else if (!this.needUpdate && debugMode) {
         console.clog(caller + " no update needed to " + aName);
+      }
       return false;
     }
   };
 
-  let fnName = aName.split(".").pop();
+  let name = afnName.split(".").pop();
   try {
-    return new ChangeCode({obj: aParent, fnName: fnName,
-      fullName: aName, options: aOptions});
+    return new ChangeCode({obj: aParent, fnName: name,
+                           fullName: afnName, options: aOptions});
   } catch (ex) {
-    console.clog(console.callerName() + " failed to change " + aName + "\nError: " + ex.message);
+    console.clog(console.callerName() + " failed to change " + afnName + "\nError: " + ex.message);
     if (debugMode)
       console.obj(aParent, "aParent");
   }
@@ -163,11 +177,23 @@ Tabmix.setNewFunction = function(aObj, aName, aCode) {
   if (!Object.getOwnPropertyDescriptor(aObj, aName)) {
     Object.defineProperty(aObj, aName, {value: aCode,
                                         writable: true, configurable: true});
-  }
-  else
+  } else {
     aObj[aName] = aCode;
+  }
 };
 
 Tabmix.nonStrictMode = function(aObj, aFn, aArg) {
   aObj[aFn].apply(aObj, aArg || []);
 };
+
+(function(obj) {
+  let global = Components.utils.getGlobalForObject(obj);
+  // eslint-disable-next-line no-useless-concat
+  let fn = global["ev" + "al"];
+  Tabmix._makeCode = function(name, code) {
+    if (name) {
+      return fn(name + " = " + code);
+    }
+    return fn("(" + code + ")");
+  };
+}(this));
