@@ -1,3 +1,4 @@
+/* exported TabmixTabClickOptions, TabmixAllTabs */
 "use strict";
 
 XPCOMUtils.defineLazyModuleGetter(Tabmix, "ContextMenu",
@@ -18,8 +19,9 @@ var TabmixTabClickOptions = {
     if (leftClick && aEvent.detail > 1) {
       if (this._tabFlipTimeOut)
         this.clearTabFlipTimeOut();
-      if (this._blockDblClick)
-        setTimeout(function(self) {self._blockDblClick = false;}, 0, this);
+      if (this._blockDblClick) {
+        setTimeout(() => (this._blockDblClick = false), 0);
+      }
       return; // double click (with left button)
     }
 
@@ -30,10 +32,10 @@ var TabmixTabClickOptions = {
     this._blockDblClick = target.classList.contains("tabs-newtab-button") ||
       !Tabmix.isVersion(270) && leftClick && anonid == "tmp-close-button";
 
-    // don't do anything if user left click on close tab button, or on any
-    // other button on tab or tabbar
-    if (leftClick && (anonid == "tmp-close-button" ||
-                      target.localName == "toolbarbutton")) {
+    // don't do anything if user left click on tab or tabbar button
+    if (leftClick &&
+        (anonid == "tmp-close-button" || aEvent.target._overPlayingIcon ||
+         target.localName == "toolbarbutton")) {
       return;
     }
 
@@ -50,14 +52,14 @@ var TabmixTabClickOptions = {
     // we replace click handler from tab binding with this to make sure that we
     // always call onMouseCommand (if we need to) before we call tab flip.
     // tabcontainer click handler run before tab click handler.
-    if (leftClick && !clickOutTabs &&  !tab.mouseDownSelect)
-      tab.onMouseCommand(aEvent);
+    if (leftClick && !clickOutTabs && !tab.mouseDownSelect)
+      tab.onMouseCommand(aEvent, true);
 
     // for tab flip
     if (!clickOutTabs && leftClick && tab.hasAttribute("clickOnCurrent")) {
       tab.removeAttribute("clickOnCurrent");
       let tabFlip = Tabmix.prefs.getBoolPref("tabFlip");
-      if (tabFlip && !aEvent.shiftKey && !aEvent.ctrlKey && !aEvent.altKey && !aEvent.metaKey){
+      if (tabFlip && !aEvent.shiftKey && !aEvent.ctrlKey && !aEvent.altKey && !aEvent.metaKey) {
         let self = this;
         let tabFlipDelay = Tabmix.prefs.getIntPref("tabFlipDelay");
         if (this._tabFlipTimeOut)
@@ -73,27 +75,21 @@ var TabmixTabClickOptions = {
     }
 
     var prefName;
-    /* middle click*/
-    if (aEvent.button == 1)
-      prefName = "middle";
-    /* shift click*/
-    else if (leftClick && aEvent.shiftKey && !aEvent.ctrlKey &&
+    if (aEvent.button == 1) {
+      prefName = "middle"; /* middle click*/
+    } else if (leftClick && aEvent.shiftKey && !aEvent.ctrlKey &&
         !aEvent.altKey && !aEvent.metaKey) {
-      prefName = "shift";
-    }
-    /* alt click*/
-    else if (leftClick && aEvent.altKey && !aEvent.ctrlKey &&
+      prefName = "shift"; /* shift click*/
+    } else if (leftClick && aEvent.altKey && !aEvent.ctrlKey &&
         !aEvent.shiftKey && !aEvent.metaKey) {
-      prefName = "alt";
-      window.addEventListener("keyup", function TMP_onKeyup_onTabClick(aEvent) {
-        aEvent.currentTarget.removeEventListener("keyup", TMP_onKeyup_onTabClick, true);
-        aEvent.stopPropagation();
+      prefName = "alt"; /* alt click*/
+      window.addEventListener("keyup", function TMP_onKeyup_onTabClick(event) {
+        event.currentTarget.removeEventListener("keyup", TMP_onKeyup_onTabClick, true);
+        event.stopPropagation();
       }, true);
-    }
-    /* ctrl click*/
-    else if (leftClick && (aEvent.ctrlKey && !aEvent.metaKey ||
+    } else if (leftClick && (aEvent.ctrlKey && !aEvent.metaKey ||
         !aEvent.ctrlKey && aEvent.metaKey) && !aEvent.shiftKey && !aEvent.altKey) {
-      prefName = "ctrl";
+      prefName = "ctrl"; /* ctrl click*/
     }
 
     if (prefName)
@@ -112,9 +108,10 @@ var TabmixTabClickOptions = {
       return;
     }
 
-    // don't do anything if user click on close tab button , or on any other button on tab or tabbar
     var target = aEvent.originalTarget;
-    if (target.getAttribute("anonid") == "tmp-close-button" ||
+    var anonid = target.getAttribute("anonid");
+    // don't do anything if user left click on tab or tabbar button
+    if (anonid == "tmp-close-button" || aEvent.target._overPlayingIcon ||
         target.localName == "toolbarbutton") {
       return;
     }
@@ -143,7 +140,7 @@ var TabmixTabClickOptions = {
     }
   },
 
-///add option to open new tab after current one
+/// add option to open new tab after current one
 /// convert this switch to object
   doCommand: function TMP_doCommand(command, aTab, clickOutTabs) {
     gBrowser.selectedBrowser.focus();
@@ -185,10 +182,12 @@ var TabmixTabClickOptions = {
       case 11:
         Tabmix.renameTab.editTitle(aTab);
         break;
-      case 12: //taken from tco
+      case 12: { // taken from tco
+        let SessionSaver = window.SessionSaver;
         if (SessionSaver && SessionSaver.snapBackTab)
           SessionSaver.snapBackTab(SessionSaver.snapback_noFX, SessionSaver.snapback_willFocus);
         break;
+      }
       case 13:
         TMP_ClosedTabs.restoreTab("original", -2);
         break;
@@ -213,13 +212,12 @@ var TabmixTabClickOptions = {
       case 20:
         gBrowser._reloadRightTabs(aTab);
         break;
-      case 21: // taken from tco
-        var href;
+      case 21: { // taken from tco
+        let href;
         if (window.IeView && window.IeView.ieViewLaunch) {
           href = gBrowser.getBrowserForTab(aTab).currentURI.spec;
-          IeView.ieViewLaunch("Internet Explorer.lnk", href);
-        }
-        else if (Tabmix.extensions.gIeTab) {
+          window.IeView.ieViewLaunch("Internet Explorer.lnk", href);
+        } else if (Tabmix.extensions.gIeTab) {
           let ieTab = Tabmix.extensions.gIeTab;
           let gIeTabObj = window[ieTab.obj];
           if (typeof gIeTabObj.switchTabEngine == "function") {
@@ -227,12 +225,12 @@ var TabmixTabClickOptions = {
               gBrowser.selectedTab = aTab;
             gIeTabObj.switchTabEngine(aTab, gIeTabObj.getBoolPref(ieTab.folder + ".alwaysNewTab", false));
           }
-        }
-        else if(window.ieview && window.ieview.launch) {
+        } else if (window.ieview && window.ieview.launch) {
           href = gBrowser.getBrowserForTab(aTab).currentURI.spec;
-          ieview.launch(href);
+          window.ieview.launch(href);
         }
         break;
+      }
       case 22:
         gBrowser.SelectToMerge(aTab);
         break;
@@ -280,7 +278,7 @@ var TabmixTabClickOptions = {
           gBrowser.pinTab(aTab);
         break;
       case 32:
-          gBrowser.previousTab(gBrowser.selectedTab);
+        gBrowser.previousTab(gBrowser.selectedTab);
         break;
       default:
         return false;
@@ -308,17 +306,21 @@ var TabmixTabClickOptions = {
 };
 
 var TabmixContext = {
-  _closeRightTabs: "tm-closeRightTabs",
   // Create new items in the tab bar context menu
   buildTabContextMenu: function TMP_buildTabContextMenu() {
-    var $id = function(id) document.getElementById(id);
+    var $id = id => document.getElementById(id);
 
     var tabContextMenu = $id("tabContextMenu");
     tabContextMenu.insertBefore($id("context_reloadTab"), $id("tm-autoreloadTab_menu"));
     tabContextMenu.insertBefore($id("context_openTabInWindow"), $id("context_pinTab"));
     tabContextMenu.insertBefore($id("context_bookmarkAllTabs"), $id("context_bookmarkTab").nextSibling);
     tabContextMenu.insertBefore($id("context_closeTab"), $id("tm-closeAllTabs"));
-    tabContextMenu.insertBefore($id("context_closeOtherTabs"), $id("tm-closeLeftTabs"));
+    let closeLeftTabs = $id("tm-closeLeftTabs");
+    let closeTabsToTheEnd = $id("context_closeTabsToTheEnd");
+    tabContextMenu.insertBefore($id("context_closeOtherTabs"), closeLeftTabs);
+    tabContextMenu.insertBefore(closeTabsToTheEnd, closeLeftTabs.nextSibling);
+    Tabmix.setItem(closeTabsToTheEnd, "oncommand", "gBrowser._closeRightTabs(TabContextMenu.contextTab);");
+
     // we can't disable menus with command attribute
     $id("context_undoCloseTab").removeAttribute("command");
 
@@ -328,8 +330,16 @@ var TabmixContext = {
         tabContextMenu.insertBefore(openNonRemote, $id("context_openTabInWindow").nextSibling);
     }
 
-    // insret IE Tab menu-items before Bookmakrs menu-items
+    if (Tabmix.isVersion(430)) {
+      let toggleMuteTab = $id("context_toggleMuteTab");
+      if (toggleMuteTab) {
+        tabContextMenu.insertBefore(toggleMuteTab, $id("context_pinTab"));
+      }
+    }
+
+    // insert IE Tab menu-items before Bookmarks menu-items
     if ("gIeTab" in window) { // no need to do this fix for IE Tab 2
+      let IeTab = window.IeTab;
       var aFunction = "createTabbarMenu" in IeTab.prototype ? "createTabbarMenu" : "init";
       if (aFunction in IeTab.prototype) {
         Tabmix.changeCode(IeTab.prototype, "IeTab.prototype." + aFunction)._replace(
@@ -340,22 +350,11 @@ var TabmixContext = {
     }
 
     // fix conflict with CookiePie extension
-    if ("cookiepieContextMenu" in window && !cookiepieContextMenu.initialized)
-      cookiepieContextMenu.init();
-
-    // Bug 866880 - Implement "Close Tabs to the Right" as a built-in feature
-    if (Tabmix.isVersion(240)) {
-      tabContextMenu.insertBefore($id("context_closeTabsToTheEnd"), $id("tm-closeRightTabs"));
-      $id("context_closeTabsToTheEnd").setAttribute("oncommand","gBrowser._closeRightTabs(TabContextMenu.contextTab);");
-      tabContextMenu.removeChild($id("tm-closeRightTabs"));
-      this._closeRightTabs = "context_closeTabsToTheEnd";
-    }
-
-    if (Tabmix._restoreMultipleTabs) {
-      let multipletablabel = $id("context_undoCloseTab").getAttribute("multipletablabel");
-      let undoCloseTabMenu = $id("tm-content-undoCloseTab");
-      undoCloseTabMenu.setAttribute("singletablabel", undoCloseTabMenu.label);
-      undoCloseTabMenu.setAttribute("multipletablabel", multipletablabel);
+    if ("cookiepieContextMenu" in window) {
+      let cookiepieContextMenu = window.cookiepieContextMenu;
+      if (!cookiepieContextMenu.initialized) {
+        cookiepieContextMenu.init();
+      }
     }
 
     if (Tabmix.prefs.getBoolPref("showTabContextMenuOnTabbar"))
@@ -371,9 +370,9 @@ var TabmixContext = {
     if (show) {
       this._originalTabbarContextMenu = tabBar.getAttribute("context");
       tabBar.setAttribute("context", gBrowser.tabContextMenu.id);
-    }
-    else
+    } else {
       Tabmix.setItem(tabBar, "context", this._originalTabbarContextMenu || null);
+    }
   },
 
   toggleEventListener: function(enable) {
@@ -387,6 +386,9 @@ var TabmixContext = {
     let id = aEvent.target.id;
     switch (aEvent.type) {
       case "popupshowing":
+        if (aEvent.target.state != "showing") {
+          return;
+        }
         if (id == "tabContextMenu")
           this.updateTabContextMenu(aEvent);
         else if (id == "contentAreaContextMenu")
@@ -427,8 +429,7 @@ var TabmixContext = {
     if (clickOutTabs) {
       Tabmix.setItem(newTab, "label", newTab.getAttribute("_newtab"));
       Tabmix.setItem(newTab, "oncommand", "TMP_BrowserOpenTab();");
-    }
-    else {
+    } else {
       Tabmix.setItem(newTab, "label", newTab.getAttribute("_newtab") + "  " + newTab.getAttribute("_afterthis"));
       Tabmix.setItem(newTab, "oncommand", "TMP_BrowserOpenTab(TabContextMenu.contextTab);");
     }
@@ -443,10 +444,19 @@ var TabmixContext = {
                       !Tabmix.singleWindowMode && gMultiProcessBrowser);
     }
 
+    if (Tabmix.isVersion(430)) {
+      Tabmix.showItem("context_toggleMuteTab", Tabmix.prefs.getBoolPref("muteTabMenu"));
+    }
+
     var show = Tabmix.prefs.getBoolPref("pinTabMenu");
     Tabmix.showItem("context_pinTab", show && !aTab.pinned);
     Tabmix.showItem("context_unpinTab", show && aTab.pinned);
-    Tabmix.showItem("context_tabViewMenu", Tabmix.prefs.getBoolPref("moveToGroup") && !aTab.pinned);
+    setTimeout(() => {
+      // we need to set our show/hide after tabGroups extension
+      let tabViewMenu = document.getElementById("context_tabViewMenu") ||
+          document.getElementById("tabGroups-context_tabViewMenu");
+      Tabmix.showItem(tabViewMenu, Tabmix.prefs.getBoolPref("moveToGroup") && !aTab.pinned);
+    });
     Tabmix.showItem("tm-mergeWindowsTab",
                     Tabmix.prefs.getBoolPref("showMergeWindow") &&
                     (!Tabmix.singleWindowMode ||
@@ -473,14 +483,18 @@ var TabmixContext = {
 
     //  ---------------- menuseparator ---------------- //
 
+    var tabsCount = gBrowser.visibleTabs.length;
+    var unpinnedTabsCount = tabsCount - TabmixTabbar._real_numPinnedTabs;
+    var unpinnedTabs = unpinnedTabsCount > 0;
+
     // Close tab Commands
     var pinnedTab = TabContextMenu.contextTab.pinned;
     Tabmix.showItem("context_closeTab", Tabmix.prefs.getBoolPref("closeTabMenu"));
-    Tabmix.showItem("tm-closeAllTabs", Tabmix.prefs.getBoolPref("closeAllMenu") && !pinnedTab);
-    Tabmix.showItem("tm-closeSimilar", Tabmix.prefs.getBoolPref("closeSimilarTabs") && !pinnedTab);
-    Tabmix.showItem("context_closeOtherTabs", Tabmix.prefs.getBoolPref("closeOtherMenu") && !pinnedTab);
+    Tabmix.showItem("tm-closeAllTabs", Tabmix.prefs.getBoolPref("closeAllMenu") && unpinnedTabs);
+    Tabmix.showItem("tm-closeSimilar", Tabmix.prefs.getBoolPref("closeSimilarTabs") && unpinnedTabs);
+    Tabmix.showItem("context_closeOtherTabs", Tabmix.prefs.getBoolPref("closeOtherMenu") && unpinnedTabs);
     Tabmix.showItem("tm-closeLeftTabs", Tabmix.prefs.getBoolPref("closeLeftMenu") && !pinnedTab);
-    Tabmix.showItem(this._closeRightTabs, Tabmix.prefs.getBoolPref("closeRightMenu") && !pinnedTab);
+    Tabmix.showItem("context_closeTabsToTheEnd", Tabmix.prefs.getBoolPref("closeRightMenu") && unpinnedTabs);
 
     //  ---------------- menuseparator ---------------- //
 
@@ -512,18 +526,18 @@ var TabmixContext = {
 
     var protectedTab = aTab.hasAttribute("protected");
     var lockedTab = aTab.hasAttribute("locked");
-    var tabsCount = gBrowser.visibleTabs.length;
-    var unpinnedTabs = tabsCount - TabmixTabbar._real_numPinnedTabs;
+    var noTabsToClose = !unpinnedTabsCount || unpinnedTabsCount == 1 && !aTab.pinned;
     var cIndex = Tabmix.visibleTabs.indexOf(aTab);
     if (Tabmix.rtl)
       cIndex = tabsCount - 1 - cIndex;
 
     var keepLastTab = tabsCount == 1 && Tabmix.prefs.getBoolPref("keepLastTab");
     Tabmix.setItem("context_closeTab", "disabled", protectedTab || keepLastTab);
-    Tabmix.setItem("tm-closeAllTabs", "disabled", keepLastTab || unpinnedTabs <= 1);
-    Tabmix.setItem("context_closeOtherTabs", "disabled", unpinnedTabs <= 1);
-    Tabmix.setItem(this._closeRightTabs, "disabled", cIndex == tabsCount - 1 || unpinnedTabs <= 1);
-    Tabmix.setItem("tm-closeLeftTabs", "disabled", cIndex === 0 || unpinnedTabs <= 1);
+    Tabmix.setItem("tm-closeAllTabs", "disabled", keepLastTab || !unpinnedTabsCount);
+    Tabmix.setItem("context_closeOtherTabs", "disabled", noTabsToClose);
+    Tabmix.setItem("context_closeTabsToTheEnd", "disabled", cIndex == tabsCount - 1 || noTabsToClose);
+    Tabmix.setItem("tm-closeLeftTabs", "disabled",
+                   cIndex === 0 || aTab.pinned || Tabmix.visibleTabs.previous(aTab).pinned);
 
     var closeTabsEmpty = TMP_ClosedTabs.count < 1;
     Tabmix.setItem("context_undoCloseTab", "disabled", closeTabsEmpty);
@@ -540,15 +554,15 @@ var TabmixContext = {
     Tabmix.setItem("tm-docShell", "disabled", clickOutTabs);
 
     var freezeTabMenu = document.getElementById("tm-freezeTab");
-    if ( !freezeTabMenu.hidden )
+    if (!freezeTabMenu.hidden)
       Tabmix.setItem(freezeTabMenu, "checked", lockedTab && protectedTab);
 
     var lockTabMenu = document.getElementById("tm-lockTab");
-    if ( !lockTabMenu.hidden )
+    if (!lockTabMenu.hidden)
       Tabmix.setItem(lockTabMenu, "checked", lockedTab);
 
     var protectTabMenu = document.getElementById("tm-protectTab");
-    if ( !protectTabMenu.hidden )
+    if (!protectTabMenu.hidden)
       Tabmix.setItem(protectTabMenu, "checked", protectedTab);
 
     return true;
@@ -565,7 +579,7 @@ var TabmixContext = {
       return;
     // don't show 2 menuseparator together
     var hideNextSeparator = true, lastVisible, hideMenu = true;
-    for(var mi = tabContextMenu.firstChild; mi; mi = mi.nextSibling) {
+    for (var mi = tabContextMenu.firstChild; mi; mi = mi.nextSibling) {
       if (mi.localName == "menuseparator") {
         if (!lastVisible || !hideNextSeparator) {
           mi.hidden = hideNextSeparator;
@@ -573,18 +587,16 @@ var TabmixContext = {
             hideNextSeparator = true;
             lastVisible = mi;
           }
-        }
-        else if (hideNextSeparator) {
-          if (lastVisible.getAttribute("type")=="tabmix" && mi.getAttribute("type")!="tabmix") {
+        } else if (hideNextSeparator) {
+          if (lastVisible.getAttribute("type") == "tabmix" && mi.getAttribute("type") != "tabmix") {
             mi.hidden = false;
             lastVisible.hidden = true;
             lastVisible = mi;
-          }
-          else
+          } else {
             mi.hidden = true;
+          }
         }
-      }
-      else if(!mi.hidden && !mi.collapsed) {
+      } else if (!mi.hidden && !mi.collapsed) {
         hideNextSeparator = false;
         hideMenu = false;
       }
@@ -604,6 +616,9 @@ var TabmixContext = {
     if (!gContextMenu || event.originalTarget != document.getElementById("contentAreaContextMenu"))
       return true;
 
+    gContextMenu.tabmixLinks = Tabmix.contextMenuLinks;
+    Tabmix.contextMenuLinks = null;
+
     var tab = gBrowser.selectedTab;
     try {
       var contentClick = gContextMenu.onTextInput || gContextMenu.onLink || gContextMenu.onImage;
@@ -615,17 +630,17 @@ var TabmixContext = {
      /**
       * from Firefox 4.0 2009-09-11 there is gContextMenu.openLinkInCurrent
       * Firefox only show this menu when the selection text is url see Bug 454518
-      * we cahck if gContextMenu.linkURL contain URL
+      * we check if gContextMenu.linkURL contain URL
       */
       var onLink = gContextMenu.onLink || gContextMenu.linkURL;
       Tabmix.showItem("context-openlinkincurrent", Tabmix.prefs.getBoolPref("openLinkHere") && onLink);
       var inverseLink = document.getElementById("tm-openinverselink");
       Tabmix.showItem(inverseLink, Tabmix.prefs.getBoolPref("openInverseLink") && onLink);
-      if (!inverseLink.hidden){
+      if (!inverseLink.hidden) {
         let bgPref = Services.prefs.getBoolPref("browser.tabs.loadInBackground");
-        let focusType = bgPref ? "fg":"bg";
-        inverseLink.setAttribute("label", inverseLink.getAttribute(focusType+"label"));
-        inverseLink.setAttribute("accesskey", inverseLink.getAttribute(focusType+"accesskey"));
+        let focusType = bgPref ? "fg" : "bg";
+        inverseLink.setAttribute("label", inverseLink.getAttribute(focusType + "label"));
+        inverseLink.setAttribute("accesskey", inverseLink.getAttribute(focusType + "accesskey"));
       }
       Tabmix.showItem("tm-linkWithhistory", Tabmix.prefs.getBoolPref("linkWithHistory") && onLink);
       var closeTabMenu = document.getElementById("tm-content-closetab");
@@ -635,12 +650,11 @@ var TabmixContext = {
 
       // for remote tab get call getValidUrl when it is safe to use CPOWs
       // getValidUrl may call getParamsForLink
-      if (tab.getAttribute("remote") == "true" &&
+      if (tab.linkedBrowser.getAttribute("remote") == "true" &&
           onLink && (Tabmix.prefs.getBoolPref("openLinkHere") ||
                      Tabmix.prefs.getBoolPref("openInverseLink") ||
                      Tabmix.prefs.getBoolPref("linkWithHistory"))) {
-        let {target, linkURL} = gContextMenu;
-        gContextMenu.tabmixLinkURL = tablib.getValidUrl(linkURL, target);
+        gContextMenu.tabmixLinkURL = tablib.getValidUrl();
       }
 
       var freezeTabMenu = document.getElementById("tm-content-freezeTab");
@@ -677,11 +691,6 @@ var TabmixContext = {
       Tabmix.showItem(undoCloseTabMenu, !contentClick &&
           !gContextMenu.isTextSelected && undoClose && !closeTabsEmpty &&
           Tabmix.prefs.getBoolPref("undoCloseTabContent"));
-      if (Tabmix._restoreMultipleTabs) {
-        let closedTabCount = TabmixSvc.ss.getNumberOfTabsClosedLast(window);
-        let visibleLabel = closedTabCount <= 1 ? "singletablabel" : "multipletablabel";
-        undoCloseTabMenu.setAttribute("label", undoCloseTabMenu.getAttribute(visibleLabel));
-      }
 
       var undoCloseListMenu = document.getElementById("tm-content-undoCloseList");
       Tabmix.showItem(undoCloseListMenu, !contentClick &&
@@ -705,16 +714,17 @@ var TabmixContext = {
       var textSep = document.getElementById("tm-content-textSep");
       undoCloseSep.hidden = undoCloseTabMenu.hidden && undoCloseListMenu.hidden ||
           gContextMenu.isTextSelected && closeTabMenu.hidden && lockTabMenu.hidden &&
-          protectTabMenu.hidden && tabsListMenu.hidden  && freezeTabMenu.hidden;
+          protectTabMenu.hidden && tabsListMenu.hidden && freezeTabMenu.hidden;
       miscSep.hidden = mergeMenu.hidden && closeTabMenu.hidden && duplicateTabMenu.hidden &&
           duplicateWinMenu.hidden && lockTabMenu.hidden && protectTabMenu.hidden &&
-          tabsListMenu.hidden  && freezeTabMenu.hidden || gContextMenu.isTextSelected;
+          tabsListMenu.hidden && freezeTabMenu.hidden || gContextMenu.isTextSelected;
       textSep.hidden = !gContextMenu.isTextSelected || mergeMenu.hidden &&
           duplicateTabMenu.hidden && duplicateWinMenu.hidden && closeTabMenu.hidden &&
           lockTabMenu.hidden && protectTabMenu.hidden && tabsListMenu.hidden &&
           freezeTabMenu.hidden && undoCloseTabMenu.hidden && undoCloseListMenu.hidden;
-
-    } catch (ex) {Tabmix.assert(ex);}
+    } catch (ex) {
+      Tabmix.assert(ex);
+    }
     return true;
   },
 
@@ -730,26 +740,8 @@ var TabmixContext = {
   },
 
   openMultipleLinks: function TMP_openMultipleLinks(check) {
-    let urls = [];
-    let browser = window.gBrowser.selectedBrowser;
-
-    function getLinks() {
-      try {
-        let handler = TabmixSvc.syncHandlers.get(browser.permanentKey);
-        let result = handler.getSelectedLinks();
-        gContextMenu.tabmixLinks = result && result.split('\n');
-      } catch(ex) {
-        Tabmix.log("unable to get syncHandlers for page " +
-                   browser.currentURI.spec + "\n" + ex);
-      }
-      return gContextMenu.tabmixLinks || [];
-    }
-
-    if (Tabmix.isVersion(320))
-      urls = gContextMenu.tabmixLinks || getLinks();
-    // getSelectedLinks was not implemented for remote tabs before Firefox 32
-    else if (browser.getAttribute("remote") != "true")
-      urls = Tabmix.ContextMenu.getSelectedLinks(content, check);
+    let urls = Tabmix.isVersion(420) ? gContextMenu.tabmixLinks :
+        Tabmix.ContextMenu.getSelectedLinks(window.content, check);
 
     if (!check && urls.length) {
       Tabmix.loadTabs(urls, false);
@@ -765,10 +757,11 @@ var TabmixAllTabs = {
   backupLabel: "",
   handleEvent: function TMP_AT_handleEvent(aEvent) {
     switch (aEvent.type) {
-      case "TabAttrModified":
+      case "TabAttrModified": {
         let tab = aEvent.target;
         this._setMenuitemAttributes(tab.mCorrespondingMenuitem, tab);
         break;
+      }
       case "TabClose":
         this._tabOnTabClose(aEvent);
         break;
@@ -778,9 +771,6 @@ var TabmixAllTabs = {
       case "DOMMenuItemInactive":
         this.updateMenuItemInactive(aEvent);
         break;
-      case "DOMMouseScroll":
-        TMP_eventListener.onTabBarScroll(aEvent);
-        /* falls through */
       case "scroll":
         this._popup._updateTabsVisibilityStatus();
         break;
@@ -798,15 +788,12 @@ var TabmixAllTabs = {
 
     for (var i = 0; i < this.childNodes.length; i++) {
       let curTab = this.childNodes[i].tab;
-      if (!curTab) // "Tab Groups" menuitem and its menuseparator
-        continue;
-      let curTabBO = curTab.boxObject;
-      if (!curTabBO) // "Tabs From Other Computers" menuitem
-        continue;
-      if (Tabmix.tabsUtils.isElementVisible(curTab))
-        this.childNodes[i].setAttribute("tabIsVisible", "true");
-      else
-        this.childNodes[i].removeAttribute("tabIsVisible");
+      if (curTab && curTab.boxObject) {
+        if (Tabmix.tabsUtils.isElementVisible(curTab))
+          this.childNodes[i].setAttribute("tabIsVisible", "true");
+        else
+          this.childNodes[i].removeAttribute("tabIsVisible");
+      }
     }
   },
 
@@ -839,13 +826,13 @@ var TabmixAllTabs = {
     if (event.target.disabled)
       return;
 
-    var tablist =  document.getElementById("tabslist");
+    var tablist = document.getElementById("tabslist");
 
     this.beforeCommonList(tablist);
     this.createCommonList(tablist, 2, side);
 
     if (tablist.hasChildNodes())
-      tablist.showPopup(event.target, -1, -1, "popup", "bottomleft","topleft");
+      tablist.showPopup(event.target, -1, -1, "popup", "bottomleft", "topleft");
   },
 
   removeTabFromList: function TMP_removeTabFromList(event, popup, aType) {
@@ -869,15 +856,16 @@ var TabmixAllTabs = {
         let item = popup.parentNode.parentNode;
         if (item.parentNode.id == "btn_tabslist")
           this.createTabsList(item, aType);
+      } else {
+        popup.hidePopup();
       }
-      else popup.hidePopup();
     }
   },
 
   // show sort/unsort tabs list popup after click on sorted tab menu
   showTabsListPopup: function TMP_showTabsListPopup(event) {
     event.stopPropagation();
-    setTimeout( function (popup){
+    setTimeout(function(popup) {
       popup.showPopup(popup.parentNode, -1, -1, "popup", "bottomleft", "topleft");
     }, 0, event.target.parentNode);
   },
@@ -935,8 +923,8 @@ var TabmixAllTabs = {
     var tabs;
     var i;
 
-    switch(aType) {
-      case 1:
+    switch (aType) {
+      case 1: {
         let TabSorting = function _tabSorting(tab, index) {
           this.Tab = tab;
           this.Index = index;
@@ -952,32 +940,35 @@ var TabmixAllTabs = {
         for (i = 0; i < tabs.length; i++)
           this.createMenuItems(popup, tabs[i].Tab, tabs[i].Index);
         break;
-      case 2:
+      }
+      case 2: {
         tabs = gBrowser.visibleTabs;
         let addToMenu = side != "right";
         for (let t = 0; t < tabs.length; t++) {
           let tab = tabs[t];
           let visible = side && Tabmix.tabsUtils.isElementVisible(tab);
           if (visible) {
-            if (tab.pinned)
-              continue;
-            else if (side == "left")
-              break;
-            addToMenu = true;
-            continue;
-          }
-          if (addToMenu)
+            if (!tab.pinned) {
+              if (side == "left") {
+                break;
+              }
+              addToMenu = true;
+            }
+          } else if (addToMenu) {
             this.createMenuItems(popup, tab, t);
+          }
         }
         break;
-      case 3:
+      }
+      case 3: {
         for (i = TMP_LastTab.tabs.length - 1; i >= 0; i--) {
           let tab = TMP_LastTab.tabs[i];
-          if (tab.hidden)
-            continue;
-          this.createMenuItems(popup, tab, i);
+          if (!tab.hidden) {
+            this.createMenuItems(popup, tab, i);
+          }
         }
         break;
+      }
     }
 
     if (this._selectedItem)
@@ -988,8 +979,8 @@ var TabmixAllTabs = {
     var popup = event.target;
     popup.removeEventListener("popupshown", this, false);
     let scrollBox = document.getAnonymousElementByAttribute(popup, "class", "popup-internal-box");
-    let items = Array.slice(popup.childNodes);
-    let element = items.indexOf(this._selectedItem) < popup.childElementCount/2 ? popup.firstChild : popup.lastChild;
+    let items = Array.prototype.slice.call(popup.childNodes);
+    let element = items.indexOf(this._selectedItem) < popup.childElementCount / 2 ? popup.firstChild : popup.lastChild;
     scrollBox.ensureElementIsVisible(element);
     scrollBox.ensureElementIsVisible(this._selectedItem);
   },
@@ -1002,7 +993,7 @@ var TabmixAllTabs = {
     mi.setAttribute("tooltiptext", tab.label + "\n" + url);
     let count = "";
     if (Tabmix.ltr) {
-      count = (value<9 ? "  " : "") + (value + 1) + ": ";
+      count = (value < 9 ? "  " : "") + (value + 1) + ": ";
       mi.setAttribute("count", count);
     }
     this._setMenuitemAttributes(mi, tab);
@@ -1021,11 +1012,11 @@ var TabmixAllTabs = {
     // for ColorfulTabs 6.0+
     if (typeof colorfulTabs == "object") {
       let rule = "none";
-      if (colorfulTabs.clrAllTabsPopPref) {
+      if (window.colorfulTabs.clrAllTabsPopPref) {
         let tabClr = TabmixSessionData.getTabValue(tab, "tabClr");
         if (tabClr)
           rule = "linear-gradient(rgba(255,255,255,.7),rgba(#1,.5),rgb(#1)),linear-gradient(rgb(#1),rgb(#1))"
-                 .replace("#1", tabClr, "g");
+                 .replace(/#1/g, tabClr);
       }
       mi.style.setProperty('background-image', rule, 'important');
     }
@@ -1041,8 +1032,7 @@ var TabmixAllTabs = {
     if (aTab.hasAttribute("busy")) {
       aMenuitem.setAttribute("busy", aTab.getAttribute("busy"));
       aMenuitem.removeAttribute("image");
-    }
-    else {
+    } else {
       aMenuitem.setAttribute("image", gBrowser.getIcon(aTab));
       aMenuitem.removeAttribute("busy");
     }
