@@ -4,7 +4,6 @@
  * original code by onemen
  */
 
-
 /**
  *
  * Fix compatibility with other extensions
@@ -18,10 +17,12 @@ var TMP_extensionsCompatibility = {
   },
 
   onContentLoaded: function TMP_EC_onContentLoaded() {
-    Tabmix.extensions = {treeStyleTab: false, tabGroupManager: false,
-        verticalTabBar: false, ieTab2: false,
-        gIeTab: false, /* for ieTab and ieTab2 */
-        ctr: false /* classic theme restorer */};
+    Tabmix.extensions = {
+      treeStyleTab: false, tabGroupManager: false,
+      verticalTabBar: false, ieTab2: false,
+      gIeTab: false, /* for ieTab and ieTab2 */
+      ctr: false/* classic theme restorer */
+    };
 
     if (typeof classicthemerestorerjs == "object") {
       Tabmix.extensions.ctr = true;
@@ -44,19 +45,21 @@ var TMP_extensionsCompatibility = {
       if ("TabGroupsManagerApiVer1" in window) {
         Tabmix.extensions.tabGroupManager = true;
         window.TMP_TabGroupsManager = {};
-        window.TMP_TabGroupsManager.tabmixSessionsManager = function () {};
+        window.TMP_TabGroupsManager.tabmixSessionsManager = function() {};
         let tmp = {};
         Components.utils.import("resource://tabmixplus/extensions/TabGroupsManager.jsm", tmp);
         tmp.TMP_TabGroupsManager.changeCode = Tabmix.changeCode;
         tmp.TMP_TabGroupsManager.init(window, gBrowser.tabContainer);
       }
-    } catch (ex) {Tabmix.assert(ex, "error in TabGroupsManager.jsm");}
+    } catch (ex) {
+      Tabmix.assert(ex, "error in TabGroupsManager.jsm");
+    }
 
     // fix for Cluster Tabs - Cluster Tab look for TM_init
     // https://addons.mozilla.org/en-US/firefox/addon/cluster-tabs-for-firefox/
-    if ("GlaxChrome" in window && typeof(window.GlaxChrome) == "object") {
+    if ("GlaxChrome" in window && typeof (window.GlaxChrome) == "object") {
       document.getElementById("main-window").setAttribute("gscltTMPinstalled", true);
-      let func = ["_setupForOtherExtensions","enableCustomDragDropMode"];
+      let func = ["_setupForOtherExtensions", "enableCustomDragDropMode"];
       let GlaxChrome = window.GlaxChrome.CLT.DragDropManager;
       func.forEach(function(aFn) {
         if (aFn in GlaxChrome) {
@@ -74,14 +77,17 @@ var TMP_extensionsCompatibility = {
     try {
       if ("TreeStyleTabService" in window) {
         this.treeStyleTab.onContentLoaded();
+        this.treeStyleTab.installed = true;
         Tabmix.extensions.treeStyleTab = true;
         Tabmix.extensions.verticalTabBar = true;
       }
-    } catch (ex) {Tabmix.assert(ex, this.treeStyleTab.errorMsg);}
+    } catch (ex) {
+      Tabmix.assert(ex, this.treeStyleTab.errorMsg);
+    }
 
     // https://addons.mozilla.org/en-US/firefox/addon/second-search/
     if ("SecondSearchBrowser" in window && SecondSearchBrowser.prototype) {
-      let func = ["canOpenNewTab","loadForSearch","checkToDoSearch"];
+      let func = ["canOpenNewTab", "loadForSearch", "checkToDoSearch"];
       let SSB = SecondSearchBrowser.prototype;
       func.forEach(function(aFn) {
         if (aFn in SSB && SSB[aFn].toString().indexOf("TM_init") != -1) {
@@ -97,16 +103,14 @@ var TMP_extensionsCompatibility = {
     // we replace it back here
     if (typeof bgSaverInit == "function" && typeof getBoolPref == "function" &&
             getBoolPref.toString().indexOf("bgSaverPref.prefHasUserValue(sName)") != -1) {
-      window.getBoolPref = function getBoolPref ( prefname, def ) {
+      window.getBoolPref = function getBoolPref(prefname, def) {
         try {
           return Services.prefs.getBoolPref(prefname);
-        }
-        catch(ex) {
+        } catch (ex) {
           try {
             return (bgSaverPref.prefHasUserValue(prefname) &&
                     bgSaverPref.getBoolPref(prefname));
-          }
-          catch(e) {}
+          } catch (e) {}
         }
         return def;
       };
@@ -116,7 +120,7 @@ var TMP_extensionsCompatibility = {
 
     // workaround for extensions that look for updateIcon
     // Favicon Picker 2
-    if (typeof(gBrowser.updateIcon) == "undefined") {
+    if (typeof (gBrowser.updateIcon) == "undefined") {
       gBrowser.updateIcon = function updateIcon (aTab) {
         var browser = gBrowser.getBrowserForTab(aTab);
         if ((browser.mIconURL || "") != aTab.getAttribute("image")) {
@@ -124,7 +128,7 @@ var TMP_extensionsCompatibility = {
             aTab.setAttribute("image", browser.mIconURL);
           else
             aTab.removeAttribute("image");
-          gBrowser._tabAttrModified(aTab);
+          gBrowser._tabAttrModified(aTab, ["image"]);
         }
       }
     }
@@ -132,7 +136,7 @@ var TMP_extensionsCompatibility = {
 
     /*
     // https://addons.mozilla.org/en-US/firefox/addon/tab-flick/
-    if ("TabFlick" in window && typeof(TabFlick.openPanel) == "function") {
+    if ("TabFlick" in window && typeof (TabFlick.openPanel) == "function") {
       Tabmix.changeCode(TMP_tabDNDObserver, "TMP_tabDNDObserver.onDragEnd")._replace(
         'gBrowser.replaceTabWithWindow(draggedTab);',
         'gBrowser.selectedTab = draggedTab; TabFlick.openPanel(aEvent);'
@@ -140,11 +144,9 @@ var TMP_extensionsCompatibility = {
     }
     */
 
-
     // https://addons.mozilla.org/en-US/firefox/addon/bug489729-disable-detach-and-t//
     // we don't need to do any changes to bug489729 extension version 1.6+
 
-
     // https://addons.mozilla.org/en-US/firefox/addon/foxtab/
     if ("foxTab" in window) {
       let loadNewInBackground = '$& var loadNewInBackground = Tabmix.prefs.getBoolPref("loadNewInBackground");';
@@ -155,7 +157,7 @@ var TMP_extensionsCompatibility = {
         '  f.gBrowser.TMP_selectNewForegroundTab(newTab, false);' +
         '  TMP_LastTab.PushSelectedTab();' +
         '}';
-      if (typeof(foxTab.openNewTab) == "function") {
+      if (typeof (foxTab.openNewTab) == "function") {
         Tabmix.changeCode(foxTab, "foxTab.openNewTab")._replace(
           '{', loadNewInBackground
         )._replace(
@@ -171,7 +173,7 @@ var TMP_extensionsCompatibility = {
           'if( !loadNewInBackground) $&'
         ).toCode();
       }
-      if (typeof(foxTab.showNewTabMessage) == "function") {
+      if (typeof (foxTab.showNewTabMessage) == "function") {
         Tabmix.changeCode(foxTab, "foxTab.showNewTabMessage")._replace(
           '{', loadNewInBackground
         )._replace(
@@ -202,15 +204,22 @@ var TMP_extensionsCompatibility = {
       Tabmix.extensions.gIeTab = {obj: "gIeTab2", folder: "ietab2"};
     else if (typeof window.gIeTab == "object")
       Tabmix.extensions.gIeTab = {obj: "gIeTab", folder: "ietab"};
+
+    // prevent faviconize use its own adjustTabstrip
+    // in Firefox 4.0 we check for faviconized tabs in TMP_TabView.firstTab
+    if ("faviconize" in window && "override" in window.faviconize) {
+      Tabmix.changeCode(TMP_TabView, "TMP_TabView.checkTabs")._replace(
+        '!tab.pinned',
+        '$& && !tab.hasAttribute("faviconized")'
+      ).toCode();
+
+      // change adjustTabstrip
+      window.faviconize.override.adjustTabstrip = function() { };
+    }
   },
 
   onWindowOpen: function TMP_EC_onWindowOpen() {
-    // https://addons.mozilla.org/EN-US/firefox/addon/vertical-tabs/
-    // https://addons.mozilla.org/EN-US/firefox/addon/side-tabs/
-    if (typeof VerticalTabs == "object" || typeof sidetabs == "object") {
-      Tabmix.extensions.verticalTabs = true;
-      Tabmix.extensions.verticalTabBar = true;
-    }
+    this.setVerticalTabs();
 
     // Look for RSS/Atom News Reader
     if ("gotoLink" in window)
@@ -222,9 +231,9 @@ var TMP_extensionsCompatibility = {
     if ("RSSTICKER" in window)
       this.RSSTICKER.init();
 
-    if ("PersonaController" in window && typeof(window.PersonaController) == "object") {
+    if ("PersonaController" in window && typeof (window.PersonaController) == "object") {
       Tabmix.changeCode(PersonaController, "PersonaController._applyPersona")._replace(
-        /(\})(\)?)$/,
+        /(})(\)?)$/,
         'if (TabmixTabbar.position == 1) {\
            gBrowser.tabContainer.style.backgroundImage = this._footer.style.backgroundImage; \
            gBrowser.tabContainer.setAttribute("persona", persona.id); \
@@ -233,7 +242,7 @@ var TMP_extensionsCompatibility = {
       ).toCode();
 
       Tabmix.changeCode(PersonaController, "PersonaController._applyDefault")._replace(
-        /(\})(\)?)$/,
+        /(})(\)?)$/,
         'if (TabmixTabbar.position == 1) {\
            gBrowser.tabContainer.style.backgroundImage = ""; \
            gBrowser.tabContainer.removeAttribute("persona"); \
@@ -243,7 +252,7 @@ var TMP_extensionsCompatibility = {
     }
 
     // Firefox sync
-    // fix bug in firefox sync that add new menu itep from each popupshowing
+    // fix bug in firefox sync that add new menu item from each popupshowing
     if ("gFxWeaveGlue" in window) {
       Tabmix.changeCode(gFxWeaveGlue, "gFxWeaveGlue.handleEvent")._replace(
         'else if (this.getPageIndex() == -1)',
@@ -266,7 +275,7 @@ var TMP_extensionsCompatibility = {
     // trigger tabmix function when user change tab width with faviconize extension
     if ("faviconize" in window && "toggle" in faviconize) {
       Tabmix.changeCode(faviconize, "faviconize.toggle")._replace(
-        /(\})(\)?)$/,
+        /(})(\)?)$/,
         '  tab.removeAttribute("minwidth");' +
         '  tab.removeAttribute("maxwidth");' +
         '  TabmixTabbar.updateScrollStatus();' +
@@ -309,7 +318,9 @@ var TMP_extensionsCompatibility = {
     try {
       if ("TreeStyleTabService" in window)
         this.treeStyleTab.onWindowLoaded();
-    } catch (ex) {Tabmix.assert(ex, this.treeStyleTab.errorMsg);}
+    } catch (ex) {
+      Tabmix.assert(ex, this.treeStyleTab.errorMsg);
+    }
 
     /* fast dial FdUtils*/
     if ("FdUtils" in window && FdUtils.whereToOpenLink) {
@@ -319,10 +330,10 @@ var TMP_extensionsCompatibility = {
       ).toCode();
     }
 
-    // for MR Tech's local install extention
-    if (typeof(Local_Install) == "object") {
-      // don't open trober in current tab when tab is locked
-      // or trober is to diffrent site then the current
+    // for MR Tech's local install extension
+    if (typeof (Local_Install) == "object") {
+      // don't open 'Throbber' in current tab when tab is locked
+      // or 'Throbber' is to different site then the current
       Tabmix.changeCode(Local_Install, "Local_Install.openThrobber")._replace(
         'local_common.openURL(local_common.getThrobberURL(), inNewTab);',
         'var url = local_common.getThrobberURL(); \
@@ -338,7 +349,7 @@ var TMP_extensionsCompatibility = {
     }
 
     if ("FireGestures" in window) {
-      // unable to close surce tab after duplicate with FireGestures esextension
+      // unable to close source tab after duplicate with FireGestures extension
       // problem fix in FireGestures 1.5.7 keep this here for users with older versions
       let performAction = FireGestures._performAction.toString();
       let codeToReplace = "gBrowser.moveTabTo(newTab, ++orgTab._tPos);";
@@ -376,11 +387,11 @@ var TMP_extensionsCompatibility = {
     // override some of All-in-One Gestures function
     // override the duplicate tab function
     if (typeof aioDupTab == 'function')
-      window.aioDupTab = function() { gBrowser.duplicateTab(gBrowser.mCurrentTab); };
+      window.aioDupTab = () => gBrowser.duplicateTab(gBrowser.mCurrentTab);
 
     // override the duplicate in new window function
     if (typeof aioDupWindow == 'function')
-      window.aioDupWindow = function() { gBrowser.duplicateTabToWindow(gBrowser.mCurrentTab); };
+      window.aioDupWindow = () => gBrowser.duplicateTabToWindow(gBrowser.mCurrentTab);
 
     // override the aioCloseWindow function
     if (typeof aioCloseWindow == 'function')
@@ -449,73 +460,84 @@ var TMP_extensionsCompatibility = {
     if (window.rdrb && typeof rdrb.cleanLink == "function") {
       Tabmix.changeCode(TabmixContext, "TabmixContext.openMultipleLinks")._replace(
         'Tabmix.loadTabs(urls, false);',
-        'urls = urls.map(function(url) rdrb.cleanLink(url));\n' +
+        'urls = urls.map(url => rdrb.cleanLink(url));\n' +
         '      $&'
       ).toCode();
     }
-  }
+  },
 
+  setVerticalTabs: function() {
+    // https://addons.mozilla.org/EN-US/firefox/addon/vertical-tabs/
+    // https://addons.mozilla.org/en-US/firefox/addon/vertical-tabs-reloaded/
+    // https://addons.mozilla.org/EN-US/firefox/addon/side-tabs/
+    // https://addons.mozilla.org/en-US/firefox/addon/tabkit-2nd-edition/
+    let isVertical = typeof VerticalTabs == "object" ||
+        typeof VerticalTabsReloaded == "object" ||
+        typeof gBrowser.tabContainer._verticalTabs == "boolean" ||
+        typeof sidetabs == "object" ||
+        typeof tabkit == "object" ||
+        typeof tabkitGlobal == "object";
+    let treeStyleTab = typeof TreeStyleTabService == "object";
+    Tabmix.extensions.verticalTabBar = isVertical || treeStyleTab;
+    Tabmix.extensions.verticalTabs = isVertical && !treeStyleTab;
+  },
 };
 
 TMP_extensionsCompatibility.RSSTICKER = {
-   init : function ()  {
-     Tabmix.changeCode(RSSTICKER, "RSSTICKER.writeFeed")._replace(
-       'tbb.setAttribute("onclick"',
-       'tbb.setAttribute("onclick", "this.onClick(event);");\
-        tbb.setAttribute("_onclick"'
-     )._replace(
-       'tbb.onContextOpen =',
-       'tbb.onContextOpen = TMP_extensionsCompatibility.RSSTICKER.onContextOpen; \
-        tbb.onClick = TMP_extensionsCompatibility.RSSTICKER.onClick; \
-        tbb._onContextOpen ='
-     ).toCode();
-   },
+  init: function() {
+    Tabmix.changeCode(RSSTICKER, "RSSTICKER.writeFeed")._replace(
+      'tbb.setAttribute("onclick"',
+      'tbb.setAttribute("onclick", "this.onClick(event);");\
+       tbb.setAttribute("_onclick"'
+    )._replace(
+      'tbb.onContextOpen =',
+      'tbb.onContextOpen = TMP_extensionsCompatibility.RSSTICKER.onContextOpen; \
+       tbb.onClick = TMP_extensionsCompatibility.RSSTICKER.onClick; \
+       tbb._onContextOpen ='
+    ).toCode();
+  },
 
-   onClick : function (event) {
-     if (event.ctrlKey) {
-       this.markAsRead(true);
-     }
-     else if ((this.parent.alwaysOpenInNewTab && (event.which == 1)) || (event.which == 2)) {
-       this.onContextOpen("tab");
-     }
-     else if (event.which == 1) {
-       this.onContextOpen();
-     }
-   },
+  onClick: function(event) {
+    if (event.ctrlKey) {
+      this.markAsRead(true);
+    } else if ((this.parent.alwaysOpenInNewTab && (event.which == 1)) || (event.which == 2)) {
+      this.onContextOpen("tab");
+    } else if (event.which == 1) {
+      this.onContextOpen();
+    }
+  },
 
-   onContextOpen : function (target) {
-     if (!target) {
-       if (Tabmix.whereToOpen(null).lock)
-         this.parent.browser.openInNewTab(this.href);
-       else
-         window.loadURI(this.href);
-     }
-     else if (target == "window") {
-       if (Tabmix.singleWindowMode)
-         this.parent.browser.openInNewTab(this.href);
-       else
-         window.open(this.href);
-     }
-     else if (target == "tab") {
-       this.parent.browser.openInNewTab(this.href);
-     }
+  onContextOpen: function(target) {
+    if (!target) {
+      if (Tabmix.whereToOpen(null).lock)
+        this.parent.browser.openInNewTab(this.href);
+      else
+        window.loadURI(this.href);
+    } else if (target == "window") {
+      if (Tabmix.singleWindowMode)
+        this.parent.browser.openInNewTab(this.href);
+      else
+        window.open(this.href);
+    } else if (target == "tab") {
+      this.parent.browser.openInNewTab(this.href);
+    }
 
-     this.markAsRead();
-   }
+    this.markAsRead();
+  }
 };
 
 // prevent Wizz RSS from load pages in locked tabs
 TMP_extensionsCompatibility.wizzrss = {
   started: null,
-  init : function ()  {
+  init: function() {
     if (this.started)
       return;
     this.started = true;
     var codeToReplace = /getContentBrowser\(\).loadURI|contentBrowser.loadURI/g;
     const newCode = "TMP_extensionsCompatibility.wizzrss.openURI";
-    var _functions = ["addFeedbase","validate","gohome","tryagain","promptStuff",
-                      "doSearch","viewLog","renderItem","playEnc","renderAllEnc","playAllEnc",
-                      "gotoLink","itemLinkClick","itemListClick"];
+    var _functions = ["addFeedbase", "validate", "gohome", "tryagain", "promptStuff",
+                      "doSearch", "viewLog", "renderItem", "playEnc", "renderAllEnc", "playAllEnc",
+                      "gotoLink", "itemLinkClick", "itemListClick"];
 
     _functions.forEach(function(_function) {
       if (_function in window)
@@ -523,7 +545,7 @@ TMP_extensionsCompatibility.wizzrss = {
     });
   },
 
-  openURI : function (uri)  {
+  openURI: function(uri) {
     var w = Tabmix.getTopWin();
     var tabBrowser = w.gBrowser;
 
@@ -531,21 +553,21 @@ TMP_extensionsCompatibility.wizzrss = {
     if (openNewTab) {
       var theBGPref = !readPref("WizzRSSFocusTab", false, 2);
       tabBrowser.loadOneTab(uri, {inBackground: theBGPref});
-    }
-    else
+    } else {
       tabBrowser.loadURI(uri);
+    }
   }
 };
 
 // prevent Newsfox from load pages in locked tabs
 TMP_extensionsCompatibility.newsfox = {
-   init : function ()  {
-      Tabmix.changeCode(window, "openNewsfox")._replace(
-         /if \(newTab\) {/,
-         'newTab = newTab || Tabmix.whereToOpen(null).lock; \
-         $&'
-      ).toCode();
-   }
+  init: function() {
+    Tabmix.changeCode(window, "openNewsfox")._replace(
+      /if \(newTab\) {/,
+      'newTab = newTab || Tabmix.whereToOpen(null).lock; \
+       $&'
+    ).toCode();
+  }
 };
 
 /**
@@ -555,10 +577,12 @@ TMP_extensionsCompatibility.newsfox = {
  *  https://addons.mozilla.org/en-US/firefox/addon/tree-style-tab/
  */
 TMP_extensionsCompatibility.treeStyleTab = {
+  installed: false,
   errorMsg: "Error in Tabmix when trying to load compatible functions with TreeStyleTab extension",
 
-  preInit: function () {
-    if (typeof TreeStyleTabWindowHelper.overrideExtensionsPreInit == "function") {
+  preInit: function() {
+    let tstHelper = TreeStyleTabWindowHelper;
+    if (typeof tstHelper.overrideExtensionsPreInit == "function") {
       // overrideExtensionsPreInit look for 'gBrowser.restoreTab' in tablib.init
       tablib._init = tablib.init;
       tablib.init = function() {
@@ -569,9 +593,37 @@ TMP_extensionsCompatibility.treeStyleTab = {
          */
       };
     }
+
+    // run our initialization function before TreeStyleTab functions:
+    // preInit and onBeforeBrowserInit
+    if (typeof tstHelper.preInit == "function") {
+      Tabmix.originalFunctions.tstHelper_preInit = tstHelper.preInit;
+      tstHelper.preInit = function() {
+        TMP_eventListener._onLoad("DOMContentLoaded");
+        let method = Tabmix.originalFunctions.tstHelper_preInit;
+        method.apply(this, arguments);
+      };
+    }
+
+    if (typeof tstHelper.onBeforeBrowserInit == "function") {
+      Tabmix.originalFunctions.tstHelper_onBeforeBrowserInit = tstHelper.onBeforeBrowserInit;
+      tstHelper.onBeforeBrowserInit = function() {
+        TMP_eventListener._onLoad("load");
+        let method = Tabmix.originalFunctions.tstHelper_onBeforeBrowserInit;
+        method.apply(this, arguments);
+      };
+    }
   },
 
-  onContentLoaded: function () {
+  onContentLoaded: function() {
+    // workaround, with version 0.15.2015061300a003855
+    // gBrowser.treeStyleTab.initTabContentsOrder throw on Firefox 41+
+    Tabmix.TST_initTabContentsOrder = function() {
+      try {
+        this.initTabContentsOrder.apply(this, arguments);
+      } catch (ex) { }
+    }.bind(gBrowser.treeStyleTab);
+
     if ("TreeStyleTabBrowser" in window) {
       let obj = TreeStyleTabBrowser.prototype;
       // we don't need this in the new version since we change the tabs-frame place
@@ -592,42 +644,6 @@ TMP_extensionsCompatibility.treeStyleTab = {
         return this.document.getAnonymousElementByAttribute(aTab, 'class', 'tab-close-button close-icon');
       };
 
-      fn = obj.initTabContentsOrderInternal;
-      if (fn.toString().indexOf("closebuttons-side") == -1) {
-        Tabmix.changeCode(obj, "TreeStyleTabBrowser.prototype.initTabContentsOrderInternal")._replace(
-          'if (this.mTabBrowser.getAttribute(this.kTAB_CONTENTS_INVERTED) == \'true\')',
-          'let button = aNamedNodes.close;\n    ' +
-          'index = nodes.indexOf(button);\n   ' +
-          'if (index > -1) {\n    ' +
-          ' let tabbar = this.mTabBrowser.tabContainer;\n   ' +
-          ' let side = tabbar.getAttribute("closebuttons-side");\n    ' +
-          ' if (side == "left") {\n   ' +
-          '   let before = nodes.indexOf(aNamedNodes.twistyAnchor);\n   ' +
-          '   if (before > -1) {\n    ' +
-          '     nodes.splice(index, 1);\n   ' +
-          '     let mOver = tabbar.mCloseButtons;\n   ' +
-          '     mOver = mOver == 4 && button.getAttribute("selected") != "true" || mOver == 2;\n    ' +
-          '     let offset = mOver ? 1 : 0;\n   ' +
-          '     nodes.splice(before + offset, 0, button);\n   ' +
-          '   }\n   ' +
-          ' }\n   ' +
-          '}\n\n    ' +
-          '$&'
-        )._replace(
-          'let key = \'initTabContentsOrderInternal_\'',
-          'let self = this;\n     ' +
-          '$&'
-        ).toCode();
-
-        let callback = function() {
-          TabmixSvc.forEachBrowserWindow(function(aWindow) {
-            aWindow.gBrowser.treeStyleTab.updateInvertedTabContentsOrder(true);
-          });
-        };
-        TabmixSvc.prefs.observe("extensions.tabmix.tabs.closeButtons", callback);
-        TabmixSvc.prefs.observe("extensions.tabmix.tabs.closeButtons.onLeft", callback);
-      }
-
       // update ordinal on previous selected tab when close tab button is on the
       // left side and CloseButtons preference is 4 - close buttons on hover
       // and active tabs
@@ -635,8 +651,9 @@ TMP_extensionsCompatibility.treeStyleTab = {
         let tab = gBrowser.tabContainer.getAttribute("closebuttons-side") == "left" &&
                   gBrowser.tabContainer.mCloseButtons == 4 &&
                   event.detail && event.detail.previousTab;
-        if (tab)
-          gBrowser.treeStyleTab.initTabContentsOrder(tab);
+        if (tab) {
+          Tabmix.TST_initTabContentsOrder(tab);
+        }
       };
       gBrowser.tabContainer.addEventListener("TabSelect", ontabselect, true);
       window.addEventListener("unload", function onunload() {
@@ -656,43 +673,52 @@ TMP_extensionsCompatibility.treeStyleTab = {
     // we removed TMP_openTabNext function 2011-11-15
     if ("TreeStyleTabWindowHelper" in window && TreeStyleTabWindowHelper.overrideExtensionsDelayed) {
       Tabmix.changeCode(TreeStyleTabWindowHelper, "TreeStyleTabWindowHelper.overrideExtensionsDelayed")._replace(
+        'var newTab',
+        'gContextMenu.linkURL = url;'
+      )._replace(
+        'TreeStyleTabService.readyToOpenChildTab(aTab)',
+        'TreeStyleTabService.readyToOpenChildTab(gBrowser.mCurrentTab)'
+      )._replace(
         /eval\(["|']gBrowser\.TMP_openTabNext/,
         'if (false) $&'
       ).toCode();
     }
   },
 
-  onWindowLoaded: function () {
-    /**
-     *  TST have eval to TMP_Bookmark.openGroup
-     *  we replace TMP_Bookmark.openGroup with TMP_Places.openGroup at Tabmix 0.3.8.2pre.090830
-     *  we also replace call to TreeStyleTabService.openGroupBookmarkBehavior();
-     *  with aOpenGroupBookmarkBehavior that we pass from PlacesUIUtils._openTabset
-     *  we only call this functiom from browserWindow so we don't need to call it for
-     *  other places windows
-     */
-    Tabmix.changeCode(TMP_Places, "TMP_Places.openGroup")._replace(
-      'var tabs = gBrowser.visibleTabs;',
-      'let TSTOpenGroupBookmarkBehavior = arguments.length > 3 && arguments[3] ||\n' +
-      '        TreeStyleTabService.openGroupBookmarkBehavior();\n' +
-      '    $&'
-    )._replace(
-      'index = prevTab._tPos + 1;',
-      '  index = gBrowser.treeStyleTab.getNextSiblingTab(gBrowser.treeStyleTab.getRootTab(prevTab));' +
-      '  if (tabToSelect == aTab) index = gBrowser.treeStyleTab.getNextSiblingTab(index);' +
-      '    index = index ? index._tPos : (prevTab._tPos + 1);'
-    )._replace(
-      'prevTab = aTab;',
-      '  $&' +
-      '  if (tabToSelect == aTab && TSTOpenGroupBookmarkBehavior & TreeStyleTabService.kGROUP_BOOKMARK_SUBTREE) {' +
-      '    TreeStyleTabService.readyToOpenChildTab(tabToSelect, true, gBrowser.treeStyleTab.getNextSiblingTab(tabToSelect));' +
-      '  }'
-    )._replace(
-      /(\})(\)?)$/,
-      '  if (TSTOpenGroupBookmarkBehavior & TreeStyleTabService.kGROUP_BOOKMARK_SUBTREE)' +
-      '    TreeStyleTabService.stopToOpenChildTab(tabToSelect);' +
-      '$1$2'
-    ).toCode();
+  onWindowLoaded: function() {
+    // we don't need this hack since treestyletab version 0.16.2016021602
+    if (typeof PlacesUIUtils.__treestyletab__openTabset != "function") {
+      /**
+       *  TST have eval to TMP_Bookmark.openGroup
+       *  we replace TMP_Bookmark.openGroup with TMP_Places.openGroup at Tabmix 0.3.8.2pre.090830
+       *  we also replace call to TreeStyleTabService.openGroupBookmarkBehavior();
+       *  with aOpenGroupBookmarkBehavior that we pass from PlacesUIUtils._openTabset
+       *  we only call this function from browserWindow so we don't need to call it for
+       *  other places windows
+       */
+      Tabmix.changeCode(TMP_Places, "TMP_Places.openGroup")._replace(
+        'var tabs = gBrowser.visibleTabs;',
+        'let TSTOpenGroupBookmarkBehavior = arguments.length > 3 && arguments[3] ||\n' +
+        '        TreeStyleTabService.openGroupBookmarkBehavior();\n' +
+        '    $&'
+      )._replace(
+        'index = prevTab._tPos + 1;',
+        '  index = gBrowser.treeStyleTab.getNextSiblingTab(gBrowser.treeStyleTab.getRootTab(prevTab));' +
+        '  if (tabToSelect == aTab) index = gBrowser.treeStyleTab.getNextSiblingTab(index);' +
+        '    index = index ? index._tPos : (prevTab._tPos + 1);'
+      )._replace(
+        'prevTab = aTab;',
+        '  $&' +
+        '  if (tabToSelect == aTab && TSTOpenGroupBookmarkBehavior & TreeStyleTabService.kGROUP_BOOKMARK_SUBTREE) {' +
+        '    TreeStyleTabService.readyToOpenChildTab(tabToSelect, true, gBrowser.treeStyleTab.getNextSiblingTab(tabToSelect));' +
+        '  }'
+      )._replace(
+        /(})(\)?)$/,
+        '  if (TSTOpenGroupBookmarkBehavior & TreeStyleTabService.kGROUP_BOOKMARK_SUBTREE)' +
+        '    TreeStyleTabService.stopToOpenChildTab(tabToSelect);' +
+        '$1$2'
+      ).toCode();
+    }
 
     if (Services.prefs.getBoolPref("extensions.treestyletab.compatibility.TMP")) {
       // Added 2010-04-10
@@ -701,13 +727,13 @@ TMP_extensionsCompatibility.treeStyleTab = {
         'TabmixSessionManager.updateTabProp(aTab);',
         '$& \
          gBrowser.treeStyleTab.initTabAttributes(aTab);\
-         gBrowser.treeStyleTab.initTabContentsOrder(aTab);'
+         Tabmix.TST_initTabContentsOrder(aTab);'
       ).toCode();
       // Added 2010-04-10
       Tabmix.changeCode(TMP_eventListener, "TMP_eventListener.onTabOpen")._replace(
-        /(\})(\)?)$/,
+        /(})(\)?)$/,
         'gBrowser.treeStyleTab.initTabAttributes(tab); \
-         gBrowser.treeStyleTab.initTabContentsOrder(tab); \
+         Tabmix.TST_initTabContentsOrder(tab); \
          $1$2'
       ).toCode();
       // Added 2011-11-09, i'm not sure we really need it, Tabmix.loadTabs call gBrowser.loadTabs
@@ -716,10 +742,76 @@ TMP_extensionsCompatibility.treeStyleTab = {
         'TreeStyleTabService.readyToOpenChildTab(gBrowser, true); $1 TreeStyleTabService.stopToOpenChildTab(gBrowser);'
       ).toCode();
     }
+  },
 
-    Tabmix.changeCode(window, "window.TMP_BrowserOpenTab")._replace(
-      'var newTab = gBrowser.addTab',
-      'gBrowser.treeStyleTab.onBeforeNewTabCommand();\n   $&'
-    ).toCode();
-  }
+  onBeforeNewTabCommand: function(tab, openTabNext) {
+    if (!this.installed) {
+      return;
+    }
+    if (openTabNext) {
+      this.openNewTabNext(tab, true);
+    } else {
+      gBrowser.treeStyleTab.onBeforeNewTabCommand();
+    }
+  },
+
+  // Don't call openNewTabNext if treeStyleTab already set readiedToAttachNewTab
+  checkToOpenTabNext: function(tab, check) {
+    if (this.installed && check &&
+        !gBrowser.treeStyleTab.checkToOpenChildTab(tab)) {
+      this.openNewTabNext(tab, true);
+    }
+  },
+
+  // instruct treeStyleTab to use 'kNEWTAB_OPEN_AS_NEXT_SIBLING' when our preference
+  // is to open the tab next
+  openNewTabNext: function(tab, openTabNext, clean) {
+    if (!this.installed || !openTabNext) {
+      return;
+    }
+
+    let tst = gBrowser.treeStyleTab;
+    let browser = tst.getBrowserFromTabBrowserElements(tab);
+    if (!browser) {
+      return;
+    }
+    let ownerBrowser = tst.getTabBrowserFromChild(browser);
+    let baseTab = tst.getTabFromBrowser(browser, ownerBrowser);
+    let parentTab = tst.getParentTab(baseTab);
+
+    // clean previously ready state set by treeStyleTab
+    if (clean) {
+      tst.stopToOpenChildTab(baseTab);
+      if (parentTab) {
+        tst.stopToOpenChildTab(parentTab);
+      }
+    }
+
+    // based on treeStyleTab.readyToOpenNextSiblingTabNow
+    // we also set ready state for pinned tabs
+    let readyToOpenNextSiblingTab = function() {
+      if (!baseTab)
+        return false;
+      let nextTab = tst.getNextSiblingTab(baseTab);
+      if (parentTab) {
+        return tst.readyToOpenChildTab(parentTab, false, nextTab);
+      } else if (nextTab) {
+        ownerBrowser.treeStyleTab.readiedToAttachNewTab = true;
+        ownerBrowser.treeStyleTab.parentTab = null;
+        ownerBrowser.treeStyleTab.insertBefore = nextTab.getAttribute(tst.kID);
+        return true;
+      }
+      return false;
+    };
+
+    if (readyToOpenNextSiblingTab()) {
+      setTimeout(() => {
+        try {
+          tst.stopToOpenChildTab(baseTab);
+        } catch (ex) {
+          tst.defaultErrorHandler(ex);
+        }
+      }, 0);
+    }
+  },
 };
