@@ -5,7 +5,7 @@ Tabmix.contentAreaClick = {
     // this getter trigger by call to isGreasemonkeyInstalled from
     // TMP_extensionsCompatibility.onDelayedStartup
     XPCOMUtils.defineLazyGetter(Tabmix, "ContentClick", function() {
-      let tmp = { };
+      let tmp = {};
       Cu.import("resource://tabmixplus/ContentClick.jsm", tmp);
       return tmp.TabmixContentClick;
     });
@@ -24,7 +24,6 @@ Tabmix.contentAreaClick = {
       'loadURI(',
       'if (where == "tab" || where == "tabshifted") {\n' +
       '        let doc = event.target.ownerDocument;\n' +
-      '        let _url = Tabmix.isVersion(190) ? href : url;\n' +
       '        let params = { charset: doc.characterSet, initiatingDoc: doc,\n' +
       '                       suppressTabsOnFileDownload: suppressTabsOnFileDownload,\n' +
       '                       referrerURI: doc.documentURIObject };\n' +
@@ -32,14 +31,14 @@ Tabmix.contentAreaClick = {
       '          params.referrerPolicy = doc.referrerPolicy;\n' +
       '          params.noReferrer = BrowserUtils.linkHasNoReferrer(linkNode);\n' +
       '        }\n' +
-      '        openLinkIn(_url, where, params);\n' +
+      '        openLinkIn(href, where, params);\n' +
       '      }\n' +
       '      else\n        $&'
     )._replace(
       // force handleLinkClick to use openLinkIn by replace "current"
       // with " current", we later use trim() before handleLinkClick call openLinkIn
       'handleLinkClick(event, href, linkNode);',
-      'event.__where = where == "current" && href.indexOf("custombutton://") != 0 ? " " + where : where;\n' +
+      'event.__where = where == "current" && !href.startsWith("custombutton://") ? " " + where : where;\n' +
       '  event.__suppressTabsOnFileDownload = suppressTabsOnFileDownload;\n' +
       '  var result = $&\n' +
       '  if (targetAttr && !result)\n' +
@@ -51,8 +50,8 @@ Tabmix.contentAreaClick = {
      * Tabmix.contentAreaClick.contentLinkClick
      */
     Tabmix.changeCode(window, "handleLinkClick")._replace(
-      '{', '{\n'+
-      '  if (arguments.length > 3)\n'+
+      '{', '{\n' +
+      '  if (arguments.length > 3)\n' +
       '    event.__where = arguments[3] && arguments[3].where;'
     )._replace(
       'whereToOpenLink(event);',
