@@ -1,6 +1,13 @@
+/* globals dialog, Ci */
 "use strict";
 
-let TabmixRemoveBlankTab = {
+const Cu = Components.utils;
+
+Cu.import("resource://gre/modules/XPCOMUtils.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "Services",
+  "resource://gre/modules/Services.jsm");
+
+var TabmixRemoveBlankTab = {
   initialize: function() {
     switch (window.document.documentElement.id) {
       case "unknownContentType":
@@ -39,7 +46,8 @@ let TabmixRemoveBlankTab = {
     *   The nsIInterfaceRequestor of the parent window; may be null
     */
     let {win, b} = this.getWindowAndBrowser(window.arguments[9]);
-    if (b && b.currentURI.spec == "about:blank") {
+    let blank = "about:blank";
+    if (b && b.currentURI.spec == blank) {
       let uri = window.arguments[8].QueryInterface(Ci.nsIURI);
       if (b.userTypedValue == uri.spec) {
         let tab = win.gBrowser.getTabForBrowser(b);
@@ -59,7 +67,7 @@ let TabmixRemoveBlankTab = {
       let doc;
       try {
         doc = nav.document;
-      } catch(ex) {
+      } catch (ex) {
         return result;
       }
       result.win = nav.QueryInterface(Ci.nsIDocShellTreeItem)
@@ -67,7 +75,15 @@ let TabmixRemoveBlankTab = {
                     .QueryInterface(Ci.nsIInterfaceRequestor)
                     .getInterface(Ci.nsIDOMWindow)
                     .wrappedJSObject;
-      result.b = result.win.gBrowser.getBrowserForDocument(doc);
+      let tabBrowser = result.win.gBrowser;
+      result.b = tabBrowser.getBrowserForDocument(doc);
+      if (!result.b) {
+        // try to find tab with _tabmix_downloadingTimeout
+        let tabs = Array.prototype.filter.call(tabBrowser.tabs, t => t._tabmix_downloadingTimeout);
+        if (tabs.length) {
+          result.b = tabs[0].linkedBrowser;
+        }
+      }
     }
     return result;
   },
@@ -77,8 +93,17 @@ let TabmixRemoveBlankTab = {
       aEvent.currentTarget.removeEventListener("unload", _unload, false);
       if (win && !win.closed) {
         win.setTimeout(function() {
-          if (win && win.gBrowser && tab && tab.parentNode)
-            win.gBrowser.removeTab(tab, {animate: false});
+          let tabBrowser = win && win.gBrowser;
+          if (!tabBrowser || !tab || !tab.parentNode) {
+            return;
+          }
+          // don't remove the tab if it going to close the window
+          let closeWindow = tabBrowser.tabs.length - tabBrowser._removingTabs.length == 1 &&
+              Services.prefs.getBoolPref("browser.tabs.closeWindowWithLastTab");
+          if (closeWindow) {
+            return;
+          }
+          tabBrowser.removeTab(tab, {animate: false});
         }, 250);
       }
     }, false);
