@@ -1,5 +1,7 @@
   "use strict";
 
+  TMP_TabView.subScriptLoaded = true;
+
   TMP_TabView.handleEvent = function(aEvent) {
     switch (aEvent.type) {
       case "tabviewshown":
@@ -9,27 +11,30 @@
         TabmixSessionManager.saveTabViewData(TabmixSessionManager.gThisWin, true);
         TMP_LastTab.tabs = null;
         if (TabmixTabbar.hideMode != 2)
-          setTimeout(function () {gBrowser.tabContainer.adjustTabstrip()}, 0);
+          setTimeout(() => gBrowser.tabContainer.adjustTabstrip(), 0);
         break;
       case "TabShow":
         if (!gBrowser.tabContainer._onDelayTabShow) {
           // pass aEvent to this function for use in TGM
-          gBrowser.tabContainer._onDelayTabShow = window.setTimeout(function (aEvent) {
+          gBrowser.tabContainer._onDelayTabShow = window.setTimeout(function(event) {
             gBrowser.tabContainer._onDelayTabShow = null;
-            TMP_eventListener.onTabOpen_delayUpdateTabBar(aEvent.target);
+            TMP_eventListener.onTabOpen_delayUpdateTabBar(event.target);
           }, 0, aEvent);
         }
         break;
       case "TabHide":
         if (!gBrowser.tabContainer._onDelayTabHide) {
           // pass aEvent to this function for use in TGM
-          gBrowser.tabContainer._onDelayTabHide = window.setTimeout(function (aEvent) {
+          gBrowser.tabContainer._onDelayTabHide = window.setTimeout(function(event) {
             gBrowser.tabContainer._onDelayTabHide = null;
-            let tab = aEvent.target;
+            let tab = event.target;
             TMP_eventListener.onTabClose_updateTabBar(tab);
           }, 0, aEvent);
         }
         break;
+      case "willshowtabview":
+        Tabmix.slideshow.cancel();
+        break;
     }
   };
 
@@ -41,6 +46,11 @@
    */
 
   TMP_TabView._patchBrowserTabview = function SM__patchBrowserTabview() {
+    if (TabView.hasOwnProperty("tabmixInitialized")) {
+      return;
+    }
+    TabView.tabmixInitialized = true;
+
     var tabView = document.getElementById("tab-view-deck");
     if (tabView) {
       tabView.addEventListener("tabviewhidden", this, true);
@@ -50,11 +60,20 @@
     }
 
     // we need to stop tabs slideShow before Tabview starts
-    Tabmix.changeCode(TabView, "TabView.toggle")._replace(
-      'this.show();',
-      '{if (Tabmix.SlideshowInitialized && Tabmix.flst.slideShowTimer) Tabmix.flst.cancel();\
-       $&}'
-    ).toCode();
+    if (window.hasOwnProperty("tabGroups")) {
+      // Tab Groups extension by Quicksaver
+      // https://addons.mozilla.org/en-US/firefox/addon/tab-groups-panorama/
+      window.addEventListener("willshowtabview", this, true);
+    } else {
+      let $LF = '\n      ';
+      Tabmix.changeCode(TabView, "TabView.toggle")._replace(
+        'this.show();',
+        '{' + $LF +
+        '  Tabmix.slideshow.cancel();' + $LF +
+        '  $&' + $LF +
+        '}'
+      ).toCode();
+    }
 
     // don't do anything if Session Manager extension installed
     if (Tabmix.extensions.sessionManager)
@@ -66,7 +85,9 @@
       try {
         TabmixSessionManager._groupItemPushAway();
         this._patchTabviewFrame();
-      } catch (ex) {Tabmix.assert(ex);}
+      } catch (ex) {
+        Tabmix.assert(ex);
+      }
     }.bind(this);
 
     if (TabView._window)
@@ -77,42 +98,55 @@
 
   TMP_TabView._patchInitialized = false;
   TMP_TabView._patchTabviewFrame = function SM__patchTabviewFrame() {
+    // Tab Groups extension by Quicksaver includes these changes in its code
+    if (window.hasOwnProperty("tabGroups")) {
+      return;
+    }
+
     this._patchInitialized = true;
-    // Firefox 8.0 use strict mode - we need to map global variable
     TabView._window.GroupItems._original_reconstitute = TabView._window.GroupItems.reconstitute;
-    Tabmix.changeCode(TabView._window.GroupItems, "TabView._window.GroupItems.reconstitute")._replace(
-      '"use strict";',
-      '$&' +
-      'let win = TabView._window;' +
-      'let GroupItem = win.GroupItem;' +
-      'let iQ = win.iQ;' +
-      'let UI = win.UI;' +
-      'let Utils = win.Utils;' +
-      'let GroupItems = win.GroupItems;' +
-      'let Storage = win.Storage;', {silent: true}
-    )._replace(
-      'this.',
-      'GroupItems.', {flags: "g"}
-    )._replace(
-      // This group is re-used by session restore
-      // make sure all of its children still belong to this group.
-      // Do it before setBounds trigger data save that will overwrite
-      // session restore data.
-      // We call TabItems.resumeReconnecting later to reconnect the tabItem.
-      'groupItem.userSize = data.userSize;',
-      'groupItem.getChildren().forEach(function TMP_GroupItems_reconstitute_groupItem_forEach(tabItem) {' +
-      '  var tabData = TabmixSessionData.getTabValue(tabItem.tab, "tabview-tab", true);' +
-      '  if (!tabData || tabData.groupID != data.id) {' +
-      '    tabItem._reconnected = false;' +
-      '  }' +
-      '});' +
-      '$&'
-    ).toCode();
+    TabView._window.GroupItems.reconstitute = function(groupItemsData, groupItemData) {
+      let validate = function(groupItem, data) {
+        // This group is re-used by session restore
+        // make sure all of its children still belong to this group.
+        // Do it before setBounds trigger data save that will overwrite
+        // session restore data.
+        // TabView will use TabItems.resumeReconnecting or UI.reset to reconnect the tabItem.
+        groupItem.getChildren().forEach(tabItem => {
+          let tabData = TabmixSessionData.getTabValue(tabItem.tab, "tabview-tab", true);
+          if (!tabData || tabData.groupID != data.id) {
+            tabItem._reconnected = false;
+          }
+        });
+      };
+      if (groupItemData) {
+        let storageSanity = this.groupItemStorageSanity || this.storageSanityGroupItem;
+        let ids = Object.keys(groupItemData);
+        for (let id of ids) {
+          let data = groupItemData[id];
+          if (storageSanity(data)) {
+            let groupItem = this.groupItem(data.id);
+            if (groupItem && !groupItem.hidden) {
+              validate(groupItem, data);
+            }
+          }
+        }
+      }
+      this._original_reconstitute.apply(this, arguments);
+    };
+
+    if (window.hasOwnProperty("tabGroups")) {
+      // reconnect tabs that we disconnected in reconstitute
+      TabView._window.UI.reset = function() {
+        TabItems.resumeReconnecting();
+        this._original_reset.apply(this, arguments);
+      };
+      return;
+    }
 
     // add tab to the new group on tabs order not tabItem order
-    TabView._window.UI._original_reset = TabView._window.UI.reset;
     Tabmix.changeCode(TabView._window.UI, "TabView._window.UI.reset")._replace(
-      '"use strict";',
+      '{',
       '$&' +
       'let win = TabView._window;' +
       'let Trenches = win.Trenches;' +
@@ -121,86 +155,63 @@
       'let Rect = win.Rect;' +
       'let GroupItems = win.GroupItems;' +
       'let GroupItem = win.GroupItem;' +
+      'let TabItems = win.TabItems;' +
       'let UI = win.UI;', {silent: true}
     )._replace(
       'this.',
       'UI.', {flags: "g", silent: true}
     )._replace(
-      'items = TabItems.getItems();',
-      'items = gBrowser.tabs;'
-    )._replace(
-      /items\.forEach\(function\s*\(item\)\s*{/,
-      'Array.forEach(items, function(tab) { \
+      /items\.forEach\(function\s*\(item\)\s*\{/,
+      'Array.prototype.forEach.call(gBrowser.tabs, function(tab) { \
        if (tab.pinned) return;\
        let item = tab._tabViewTabItem;'
     )._replace(
-      'groupItem.add(item, {immediately: true});',
+      /groupItem.add\(.*\);/,
       'item._reconnected = true; \
        $&'
-    )._replace(
-      /(\})(\)?)$/,
-      '  GroupItems.groupItems.forEach(function(group) {' +
-      '    if (group != groupItem)' +
-      '      group.close();' +
-      '  });' +
-      ' $1$2'
     ).toCode();
-
-    TabView._window.TabItems._original_resumeReconnecting = TabView._window.TabItems.resumeReconnecting;
-    TabView._window.TabItems.resumeReconnecting = function TabItems_resumeReconnecting() {
-      let TabItems = TabView._window.TabItems;
-      let Utils = TabView._window.Utils;
-      Utils.assertThrow(TabItems._reconnectingPaused, "should already be paused");
-      TabItems._reconnectingPaused = false;
-      Array.forEach(gBrowser.tabs, function (tab){
-        if (tab.pinned)
-          return;
-        let item = tab._tabViewTabItem;
-        if ("__tabmix_reconnected" in item && !item.__tabmix_reconnected) {
-          item._reconnected = false;
-          delete item.__tabmix_reconnected;
-        }
-        if (!item._reconnected)
-          item._reconnect();
-      });
-    };
   };
 
-  TMP_TabView._resetTabviewFrame = function SM__resetTabviewFrame(){
+  TMP_TabView._resetTabviewFrame = function SM__resetTabviewFrame() {
     var tabView = document.getElementById("tab-view-deck");
     if (tabView) {
-      tabView.removeEventListener("tabviewhidden", this, false);
-      tabView.removeEventListener("tabviewshown", this, false);
+      tabView.removeEventListener("tabviewhidden", this, true);
+      tabView.removeEventListener("tabviewshown", this, true);
       gBrowser.tabContainer.removeEventListener("TabShow", this, true);
       gBrowser.tabContainer.removeEventListener("TabHide", this, true);
     }
+    window.removeEventListener("willshowtabview", this, true);
 
     if (this._patchInitialized && TabView._window) {
       TabView._window.GroupItems.reconstitute = TabView._window.GroupItems._original_reconstitute;
       delete TabView._window.GroupItems._original_reconstitute;
       TabView._window.UI.reset = TabView._window.UI._original_reset;
-      TabView._window.TabItems.resumeReconnecting = TabView._window.TabItems._original_resumeReconnecting;
       delete TabView._window.UI._original_reset;
-      delete TabView._window.TabItems._original_resumeReconnecting;
     }
   };
 
   /* ............... TabmixSessionManager TabView Data ............... */
 
-  // aWindow: rdfNodeWindow to read from
-  TabmixSessionManager._setWindowStateBusy = function SM__setWindowStateBusy(aWindow) {
-    TMP_SessionStore.initService();
-    this._sendWindowStateEvent("Busy");
-    this._getSessionTabviewData(aWindow);
+  // winData: SessionStore window state
+  TabmixSessionManager._setWindowStateBusy = function(winData) {
+    this._beforeRestore(winData);
+    if (!this.tabViewInstalled) {
+      return;
+    }
 
-    // save group count before we start the restore
-    var parsedData = TabmixSessionData.getWindowValue(window, "tabview-groups", true);
-    this._groupCount = parsedData.totalNumber || 1;
+    this._getSessionTabviewData(winData);
     this._updateUIpageBounds = false;
   };
 
-  TabmixSessionManager._aftertWindowStateReady =
-        function SM__aftertWindowStateReady(aOverwriteTabs, showNotification) {
+  TabmixSessionManager._setWindowStateReady = function(aOverwriteTabs, showNotification, tabsRemoved) {
+    if (Tabmix.isVersion(350)) {
+      TabmixSvc.SessionStore._setWindowStateReady(window);
+    }
+    if (!this.tabViewInstalled) {
+      this.notifyAboutMissingTabView(tabsRemoved);
+      return;
+    }
+
     if (!aOverwriteTabs)
       this._groupItems = this._tabviewData["tabview-group"];
 
@@ -215,7 +226,7 @@
 
     // update page bounds when we overwrite tabs
     if (aOverwriteTabs || this._updateUIpageBounds)
-      this._setUIpageBounds();
+      this._setPageBounds();
 
     if (TabView._window && !aOverwriteTabs) {
       // when we don't overwriting tabs try to rearrange the groupItems
@@ -229,15 +240,16 @@
 
   TabmixSessionManager.groupUpdates = {};
   TabmixSessionManager._tabviewData = {};
-  TabmixSessionManager._groupItems=  null;
+  TabmixSessionManager._groupItems = null;
 
-  // aWindow: rdfNodeWindow to read from
-  TabmixSessionManager._getSessionTabviewData = function SM__getSessionTabviewData(aWindow) {
-    let self = this;
+  // winData: SessionStore window state
+  TabmixSessionManager._getSessionTabviewData = function(winData) {
+    let extData = winData.extData || {};
     function _fixData(id, parse, def) {
-      let data = self.getLiteralValue(aWindow, id);
-      if (data && data != "null")
+      let data = extData[id] || null;
+      if (data) {
         return parse ? TabmixSvc.JSON.parse(data) : data;
+      }
       return def;
     }
 
@@ -253,13 +265,17 @@
   };
 
   TabmixSessionManager._saveTabviewData = function SM__saveTabviewData() {
+    if (!this.tabViewInstalled) {
+      return;
+    }
+
     for (let id of Object.keys(this._tabviewData)) {
       this._setTabviewData(id, this._tabviewData[id]);
     }
   };
 
   TabmixSessionManager._setTabviewData = function SM__setTabviewData(id, data) {
-    if (typeof(data) != "string")
+    if (typeof (data) != "string")
       data = TabmixSvc.JSON.stringify(data);
     TabmixSvc.ss.setWindowValue(window, id, data);
     if (!this.enableBackup)
@@ -270,13 +286,13 @@
       this.removeAttribute(this.gThisWin, id);
   };
 
-  TabmixSessionManager._setTabviewTab = function SM__setTabviewTab(aTab, tabdata, activeGroupId){
-    if (tabdata.pinned)
+  TabmixSessionManager._setTabviewTab = function SM__setTabviewTab(aTab, tabdata, activeGroupId) {
+    if (!this.tabViewInstalled || tabdata.pinned)
       return;
 
     let parsedData;
-    function setData(id) {
-      let data = { groupID: id };
+    function setData(groupID) {
+      let data = {groupID: groupID};
       parsedData = data;
       return TabmixSvc.JSON.stringify(data);
     }
@@ -286,10 +302,9 @@
     var tabviewData;
     if (update.newGroupID) {
       // We are here only when the restored session did not have tabview data
-      // we creat new group and fill all the data
+      // we create new group and fill all the data
       tabviewData = setData(update.newGroupID);
-    }
-    else {
+    } else {
       tabviewData = tabdata.extData && tabdata.extData["tabview-tab"] || null;
       // make sure data is not "null"
       if (!tabviewData || tabviewData == "null") {
@@ -331,9 +346,9 @@
         if (parsedData.groupID != activeGroupId)
           tabdata.hidden = true;
       }
-    }
-    else if (tabdata.extData)
+    } else if (tabdata.extData) {
       delete tabdata.extData["tabview-tab"];
+    }
   };
 
   TabmixSessionManager.isEmptyObject = function SM_isEmptyObject(obj) {
@@ -345,11 +360,9 @@
     if (!excludeTabs)
       excludeTabs = [];
 
-    return !Array.some(gBrowser.tabs, function(tab){
-      if (!tab.pinned && !tab.hidden && !tab.closing && excludeTabs.indexOf(tab) == -1) {
-        return true;
-      }
-      return false;
+    return !Array.prototype.some.call(gBrowser.tabs, function(tab) {
+      return !tab.pinned && !tab.hidden && !tab.closing &&
+          excludeTabs.indexOf(tab) == -1;
     });
   };
 
@@ -359,8 +372,8 @@
       aGroupsData.activeGroupId = groupID;
       this._lastSessionGroupName = "";
     }
-    let bounds = {left:0, top:0, width:350, height:300};
-    aGroupItems[groupID] = {bounds:bounds, userSize:null, title:"", id:groupID, newItem: true};
+    let bounds = {left: 0, top: 0, width: 350, height: 300};
+    aGroupItems[groupID] = {bounds: bounds, userSize: null, title: "", id: groupID, newItem: true};
     aGroupsData.totalNumber = Object.keys(aGroupItems).length;
     this._tabviewData["tabview-group"] = aGroupItems;
     this._tabviewData["tabview-groups"] = aGroupsData;
@@ -385,7 +398,7 @@
     let keys = Object.keys(aGroupItems);
     if (!aGroupsData.nextID) {
       let nextID = 0;
-      keys.forEach(function (key) {
+      keys.forEach(function(key) {
         nextID = Math.max(aGroupItems[key].id, nextID);
       });
       aGroupsData.nextID = nextID++;
@@ -397,11 +410,15 @@
   };
 
  /**
-  * when we append tab to this window we merge group data from the session into the curent group data
+  * when we append tab to this window we merge group data from the session into the current group data
   * loadOnStartup: array of tabs that load on startup from application
   * blankTabs: remaining blank tabs in this windows
   */
-  TabmixSessionManager._preperTabviewData = function SM__preperTabviewData(loadOnStartup, blankTabs) {
+  TabmixSessionManager._prepareTabviewData = function SM__prepareTabviewData(loadOnStartup, blankTabs) {
+    if (!this.tabViewInstalled) {
+      return;
+    }
+
     let newGroupItems = this._tabviewData["tabview-group"];
     let groupItems = TabmixSessionData.getWindowValue(window, "tabview-group", true);
     let newGroupItemsIsEmpty = this.isEmptyObject(newGroupItems);
@@ -434,15 +451,15 @@
 
         // update tabs data
         let groupID = newGroupsData.activeGroupId;
-        Array.forEach(gBrowser.tabs, function(tab){
+        for (let tab of gBrowser.tabs) {
           if (tab.pinned || tab.hidden || tab.closing || blankTabs.indexOf(tab) > -1)
             return;
-          let data = { groupID: groupID };
+          let data = {groupID: groupID};
           data = TabmixSvc.JSON.stringify(data);
           TabmixSvc.ss.setTabValue(tab, "tabview-tab", data);
           if (this.enableBackup)
             this.setLiteral(this.getNodeForTab(tab), "tabview-tab", data);
-        }, TabmixSessionManager);
+        }
       }
       return;
     }
@@ -490,8 +507,7 @@
         groupItems[newID].id = newID;
         // we will update tabview-tab data later
         IDs[id] = newID;
-      }
-      else {
+      } else {
         groupItems[id] = newGroupItems[id];
         if (id > groupsData.nextID)
           groupsData.nextID = id;
@@ -532,15 +548,17 @@
   /* ............... TabView Code Fix  ............... */
 
   // update page bounds when we overwrite tabs
-  TabmixSessionManager._setUIpageBounds = function SM__setUIpageBounds() {
+  TabmixSessionManager._setPageBounds = function SM__setPageBounds() {
     if (TabView._window) {
-      let data = TabView._window.Storage.readUIData(window);
+      let data = TabmixSessionData.getWindowValue(window, "tabview-ui", true);
       if (this.isEmptyObject(data))
         return;
 
-      TabView._window.UI._storageSanity(data);
-      if (data && data.pageBounds)
+      let UI = TabView._window.UI;
+      let storageSanity = UI._storageSanity || UI.storageSanity;
+      if (storageSanity(data) && data.pageBounds) {
         TabView._window.UI._pageBounds = data.pageBounds;
+      }
     }
   };
 
@@ -553,8 +571,9 @@
     for (let key of Object.keys(this._groupItems)) {
       let data = this._groupItems[key];
       if (data.newItem) {
-        if (GroupItems.groupItemStorageSanity(data)) {
-          GroupItems.groupItem(data.id).pushAway(true);
+        let group = GroupItems.groupItem(data.id);
+        if (group && GroupItems.groupItemStorageSanity(data)) {
+          group.pushAway(true);
         }
       }
     }
