@@ -1,40 +1,44 @@
+/* exported gAppearancePane */
 "use strict";
 
-var gAppearancePane = { // jshint ignore:line
-  init: function () {
+var gAppearancePane = {
+  init: function() {
     var browserWindow = Tabmix.getTopWin();
     // disable options for position the tabbar and scroll mode if TreeStyleTab extension installed
     if (browserWindow.Tabmix.extensions.verticalTabBar) {
       Tabmix.setItem("treeStyleTab.msg", "hidden", null);
-      Tabmix.setItem("tabBarDisplay", "TSTinstalled", true);
+      Tabmix.setItem("tabBarDisplay", "tstInstalled", true);
       Tabmix.setItem("tabBarPosition", "disabled", true);
       Tabmix.setItem("tabsScroll", "disabled", true);
       Tabmix.setItem("maxrow", "disabled", true);
+      Tabmix.setItem("pinnedTabScroll", "disabled", true);
       Tabmix.setItem("offsetAmountToScroll", "disabled", true);
       Tabmix.setItem("scrollDelay", "disabled", true);
       Tabmix.setItem("smoothScroll", "disabled", true);
     }
 
-    if (browserWindow.Tabmix.extensions.treeStyleTab) {
-      Tabmix.setItem("treeStyleTab.bg.msg", "hidden", null);
-      window.treeStyleTab = true;
+    if (!TabmixSvc.australis) {
+      Tabmix.setItem("squaredTabs", "hidden", true);
     }
 
-    Tabmix.setItem("tabXLeft", "disabled", !browserWindow.Tabmix.defaultCloseButtons || null);
+    let disableButtonOnLefSide = !browserWindow.Tabmix.defaultCloseButtons ||
+        browserWindow.Tabmix.extensions.treeStyleTab;
+    Tabmix.setItem("tabXLeft", "disabled", disableButtonOnLefSide || null);
     Tabmix.setItem("onLeftDisabled", "hidden", browserWindow.Tabmix.defaultCloseButtons || null);
 
     // browser.allTabs.previews
-    if (Tabmix.isVersion(210) && !TabmixSvc.isPaleMoon) {
-      gPrefWindow.removeChild("pref_allTabsPpreviews");
+    if (!TabmixSvc.isPaleMoon) {
+      gPrefWindow.removeChild("pref_allTabsPreviews");
       gPrefWindow.removeChild("obs_hideAllTabsButton");
-      gPrefWindow.removeChild("allTabsPpreviews");
+      gPrefWindow.removeChild("allTabsPreviews");
     }
 
-    // rtl update
+    // rtl update position
     var direction = window.getComputedStyle($("appearance"), null).direction;
     if (direction == "rtl") {
-      let right = $("newTabButton.posiotion.right");
-      let left = $("newTabButton.posiotion.left");
+      let right = $("newTabButton.position.right");
+      // let left = $("newTabButton.position.left");
+      let left = $("newTabButton.position.left");
       [right.label, left.label] = [left.label, right.label];
 
       let tabsScroll = $("tabsScroll").firstChild.childNodes;
@@ -52,12 +56,12 @@ var gAppearancePane = { // jshint ignore:line
     var hbox = $("tabsScroll-box");
     var label = $("tabsScroll.label").boxObject.width;
     var menulist = $("tabsScroll");
-    var ident = 23; // we have class="ident"
-    if (hbox.boxObject.width > label + menulist.boxObject.width - ident) {
+    var indent = 23; // we have class="indent"
+    if (hbox.boxObject.width > label + menulist.boxObject.width - indent) {
       menulist.parentNode.removeAttribute("pack");
       menulist.parentNode.removeAttribute("class");
       hbox.setAttribute("orient", "horizontal");
-      hbox.setAttribute("align","center");
+      hbox.setAttribute("align", "center");
     }
 
     gPrefWindow.initPane("paneAppearance");
@@ -68,8 +72,8 @@ var gAppearancePane = { // jshint ignore:line
 
   tabCloseButtonChanged: function() {
     var tabCbValue = $("pref_tabCloseButton").value;
-    Tabmix.setItem("tabXdelaycheck", "hidden", tabCbValue != 2 && tabCbValue != 4);
-    Tabmix.setItem("tabXwidthBox", "hidden", tabCbValue != 5);
+    Tabmix.setItem("tabDelayCheck", "hidden", tabCbValue != 2 && tabCbValue != 4);
+    Tabmix.setItem("tabWidthBox", "hidden", tabCbValue != 5);
   },
 
   setTabCloseButtonUI: function() {
@@ -78,17 +82,17 @@ var gAppearancePane = { // jshint ignore:line
       let tabCbUI = $("tabCloseButton");
       if (tabCbUI.selectedItem.value == 5) {
         tabCbUI.value = 1;
-        Tabmix.setItem("tabXwidthBox", "hidden", true);
+        Tabmix.setItem("tabWidthBox", "hidden", true);
       }
-    }
-    else
+    } else {
       $("alltabsItem").disabled = false;
+    }
   },
 
   tabsScrollChanged: function() {
     var multiRow = $("pref_tabsScroll").value == 2;
-    $("maxbar").hidden = !multiRow;
-    $("offsetAmountToScroll").hidden = multiRow;
+    $("multi-rows").hidden = !multiRow;
+    $("one-row").hidden = multiRow;
   },
 
   tabmixCustomizeToolbar: function() {
@@ -101,7 +105,7 @@ var gAppearancePane = { // jshint ignore:line
     var buttons = ["btn_sessionmanager", "btn_undoclose", "btn_closedwindows", "btn_tabslist"];
     var onToolbar = $("onToolbar");
     var onPlate = $("onPlate");
-    for (let i = 0; i < buttons.length; ++i ) {
+    for (let i = 0; i < buttons.length; ++i) {
       let button = aWindow.document.getElementById(buttons[i]);
       let optionButton = $("_" + buttons[i]).parentNode;
       if (button)
@@ -115,7 +119,7 @@ var gAppearancePane = { // jshint ignore:line
     // Display > Tab bar
     function updateDisabledState(buttonID, itemID, aEnable) {
       let button = aWindow.document.getElementById(buttonID);
-      let enablePosition =  button && button.parentNode == aWindow.document.getElementById("TabsToolbar");
+      let enablePosition = button && button.parentNode == aWindow.document.getElementById("TabsToolbar");
       gPrefWindow.setDisabled(itemID, !enablePosition || null);
       gPrefWindow.setDisabled("obs_" + itemID, !aEnable || !enablePosition || null);
     }
@@ -128,9 +132,9 @@ var gAppearancePane = { // jshint ignore:line
     }
   },
 
-  // block width cange on instantApply
+  // block width change on instantApply
   // user is force to hit apply
-  userchangedWidth: function(item) {
+  userChangedWidth: function(item) {
     gPrefWindow.widthChanged = $("minWidth").value != $("pref_minWidth").valueFromPreferences ||
                         $("maxWidth").value != $("pref_maxWidth").valueFromPreferences;
     if (!gPrefWindow.instantApply)
