@@ -1,8 +1,9 @@
 /* global gMenuPane */
+/* exported gEventsPane */
 "use strict";
 
 var gEventsPane = {
-  init: function () {
+  init: function() {
     // for locales with long labels
     var hbox = $("focusTab-box");
     var label = $("focusTab-label").boxObject.width;
@@ -10,7 +11,7 @@ var gEventsPane = {
     if (hbox.boxObject.width > label + menulist.boxObject.width) {
       menulist.parentNode.removeAttribute("pack");
       hbox.setAttribute("orient", "horizontal");
-      hbox.setAttribute("align","center");
+      hbox.setAttribute("align", "center");
     }
 
     var browserWindow = Tabmix.getTopWin();
@@ -20,10 +21,13 @@ var gEventsPane = {
       gPrefWindow.removeChild("ctrltab.tabPreviews");
     }
 
-    this.disableInverseMiddleClick();
+    let newTabUrl = $("pref_newTabUrl");
+    newTabUrl.name = TabmixSvc.newtabUrl;
+    newTabUrl.value = newTabUrl.valueFromPreferences;
+
     this.newTabUrl($("pref_loadOnNewTab"), false, false);
-    this.disabeleRplaceLastTabWith();
-    this.disabeleShowTabList();
+    this.disableReplaceLastTabWith();
+    this.disableShowTabList();
 
     var direction = window.getComputedStyle($("paneEvents"), null).direction;
     if (direction == "rtl") {
@@ -46,7 +50,7 @@ var gEventsPane = {
     gPrefWindow.initPane("paneEvents");
   },
 
-  disabeleShowTabList: function () {
+  disableShowTabList: function() {
     var ctrlTabPv = $("pref_ctrltab.tabPreviews");
     var disableShowTabList = $("pref_ctrltab").value &&
                              ctrlTabPv && ctrlTabPv.value;
@@ -55,7 +59,7 @@ var gEventsPane = {
       gPrefWindow.setDisabled("respondToMouse", disableShowTabList);
   },
 
-  disabeleRplaceLastTabWith: function() {
+  disableReplaceLastTabWith: function() {
     // we disable replaceLastTabWith if one of this test is true
     // browser.tabs.closeWindowWithLastTab = true OR
     // extensions.tabmix.keepLastTab = true
@@ -67,38 +71,41 @@ var gEventsPane = {
   newTabUrl: function(preference, disable, setFocus) {
     var showTabUrlBox = preference.value == 4;
     var item = $(preference.id.replace("pref_", ""));
-    var idnum = item.getAttribute("idnum") || "" ;
+    var idnum = item.getAttribute("idnum") || "";
     gPrefWindow.setDisabled("newTabUrlLabel" + idnum, !showTabUrlBox || disable);
     gPrefWindow.setDisabled("newTabUrl" + idnum, !showTabUrlBox || disable);
     if (setFocus && showTabUrlBox)
       $("newTabUrl" + idnum).focus();
   },
 
-  syncFromNewTabUrlPref: function (item) {
+  syncFromNewTabUrlPref: function(item) {
     var preference = $(item.getAttribute("preference"));
     // If the pref is set to the default, set the value to ""
     // to show the placeholder text
     let value = preference.value;
-    if (value && value.toLowerCase() == "about:newtab")
+    if (value && value.toLowerCase() == TabmixSvc.aboutNewtab)
       return "";
-    return this.syncToNewTabUrlPref(value);
+    return this.syncToNewTabUrlPref(value, TabmixSvc.aboutBlank);
   },
 
-  syncToNewTabUrlPref: function (value) {
-    // If the value is "", use about:newtab.
-    if (value === "")
-      return "about:newtab";
+  syncToNewTabUrlPref: function(value, def = TabmixSvc.aboutNewtab) {
+    // If the value is "", use about:newtab or about:blank.
+    if (value === "") {
+      return def;
+    }
 
     // Otherwise, use the actual textbox value.
     return undefined;
   },
 
-  disableInverseMiddleClick: function() {
-    var val = ($("pref_opentabforLinks") || $("pref_opentabforLinks1")).value;
-    gPrefWindow.setDisabled("inverselinks", val != 2 && $("midcurrent").checked);
+  onNewTabKeyDown: function(event) {
+    // block spaces from the user to go to about:newtab preference
+    if (event.keyCode == 32) {
+      event.preventDefault();
+    }
   },
 
-  editSlideShowKey: function () {
+  editSlideShowKey: function() {
     document.documentElement.showPane($("paneMenu"));
     if (typeof gMenuPane == "object")
       gMenuPane.editSlideShowKey();
