@@ -1,11 +1,10 @@
+/* exported gLinksPane */
 "use strict";
 
 var gLinksPane = {
-  init: function () {
-    let singleWindow = $("singleWindow");
-    $("pref_singleWindow").setAttribute("onchange", "gLinksPane.singleWindow(this.value);");
-    this.singleWindow(singleWindow.checked);
-    gLinksPane.externalLinkValue($("externalLink").checked);
+  init: function() {
+    this.singleWindow($("singleWindow").checked);
+    this.externalLinkValue($("externalLink").checked);
 
     gPrefWindow.initPane("paneLinks");
   },
@@ -22,7 +21,7 @@ var gLinksPane = {
     gPrefWindow.setDisabled("obs_externalLink", !checked);
   },
 
-  updateExternalLinkCeckbox: function (external) {
+  updateExternalLinkCheckBox: function(external) {
     let preference = $(external.getAttribute("preference"));
     if (external.value == preference.value)
       return;
