@@ -1,23 +1,34 @@
+/* exported gMenuPane */
 "use strict";
 
-var gMenuPane = { // jshint ignore:line
-  init: function () {
+var gMenuPane = {
+  init: function() {
     $("pinTab").label = gPrefWindow.pinTabLabel;
     $("togglePinTab").setAttribute("label", gPrefWindow.pinTabLabel);
+    $("clearClosedTabs").setAttribute("label", TabmixSvc.getString("undoclosetab.clear.label"));
 
     var browserWindow = Tabmix.getTopWin();
-    // if Tabview exist copy its menu label
-    if (browserWindow.TMP_TabView.installed) {
-      let label = browserWindow.document.getElementById("context_tabViewMenu").getAttribute("label");
-      $("moveToGroup").label = label;
+    let $$ = id => browserWindow.document.getElementById(id);
+
+    if (Tabmix.isVersion(430)) {
+      $("muteTab").label = browserWindow.gNavigatorBundle.getString("muteTab.label") + "/" +
+        browserWindow.gNavigatorBundle.getString("unmuteTab.label");
+    } else {
+      $("muteTab").hidden = true;
     }
-    else {
+
+    // if Tabview exist copy its menu label
+    let tabViewMenu = browserWindow.TMP_TabView.installed &&
+        ($$("context_tabViewMenu") || $$("tabGroups-context_tabViewMenu"));
+    if (tabViewMenu) {
+      $("moveToGroup").label = tabViewMenu.getAttribute("label");
+    } else {
       gPrefWindow.removeChild("pref_showMoveToGroup");
       gPrefWindow.removeChild("moveToGroup");
     }
 
     if (Tabmix.isVersion(320)) {
-      let openNonRemote = browserWindow.document.getElementById("context_openNonRemoteWindow");
+      let openNonRemote = $$("context_openNonRemoteWindow");
       if (openNonRemote) {
         let item = $("openNonRemoteWindow");
         item.setAttribute("label", openNonRemote.getAttribute("label"));
@@ -30,7 +41,7 @@ var gMenuPane = { // jshint ignore:line
     }
 
     // check if bookmark item in tab context menu
-    Tabmix.setItem("bmMenu", "hidden", !(browserWindow.document.getElementById("context_bookmarkAllTabs")));
+    Tabmix.setItem("bmMenu", "hidden", !($$("context_bookmarkAllTabs")));
 
     this.setInverseLinkLabel();
 
@@ -42,7 +53,7 @@ var gMenuPane = { // jshint ignore:line
     let paneMenu = $("paneMenu");
     if (paneMenu.hasAttribute("editSlideShowKey")) {
       paneMenu.removeAttribute("editSlideShowKey");
-      setTimeout(function(self) {self.editSlideShowKey();},0, this);
+      setTimeout(() => this.editSlideShowKey(), 0);
     }
 
     gPrefWindow.initPane("paneMenu");
@@ -58,13 +69,13 @@ var gMenuPane = { // jshint ignore:line
       return;
     shortcuts.value = newValue;
     shortcuts.keys = TabmixSvc.JSON.parse(newValue);
-    let callBack = function(shortcut) shortcut.id && shortcut.valueFromPreferences(Shortcuts.keys[shortcut.id]);
+    let callBack = shortcut => shortcut.id && shortcut.valueFromPreferences(Shortcuts.keys[shortcut.id]);
     this.updateShortcuts(shortcuts, callBack);
   },
 
   _slideShow: "",
-  updateShortcuts: function (aShortcuts, aCallBack) {
-    let boxes = Array.filter(aShortcuts.childNodes, aCallBack);
+  updateShortcuts: function(aShortcuts, aCallBack) {
+    let boxes = Array.prototype.filter.call(aShortcuts.childNodes, aCallBack);
     $("shortcuts-panel").setAttribute("usedKeys", boxes.length > 0);
     if (this._slideShow != $("shortcut-group").keys.slideShow) {
       this._slideShow = $("shortcut-group").keys.slideShow;
@@ -72,14 +83,14 @@ var gMenuPane = { // jshint ignore:line
     }
   },
 
-  setSlideShowLabel: function () {
+  setSlideShowLabel: function() {
     let slideShow = $("slideShow");
     let label = slideShow.disabled ? "??" : getFormattedKey(slideShow.key);
     $("slideDelayLabel").value = slideShow.getAttribute("_label").replace("#1", label);
     gPrefWindow.setDisabled("obs_slideDelay", slideShow.disabled);
   },
 
-  editSlideShowKey: function () {
+  editSlideShowKey: function() {
     $("menu").selectedIndex = 3;
     let slideShow = $("slideShow");
     let item = $("hide-unused-shortcuts");
