@@ -1,8 +1,13 @@
+/* import-globals-from-firefox browser/content/browser/sanitizeDialog.js */
 "use strict";
 
 Tabmix.setSanitizer = {
-  init: function () {
+  init: function() {
     this.isPromptDialog = typeof window.gSanitizePromptDialog == "object";
+    let dialog = document.getElementById("SanitizeDialog");
+    ["_label", "_accesskey", "_confirm"].forEach(att => {
+      this[att] = dialog.getAttribute("tabmix" + att);
+    });
     this.addSanitizeItem();
     this.addMenuItem();
     if (this.isPromptDialog) {
@@ -14,17 +19,19 @@ Tabmix.setSanitizer = {
     }
   },
 
-  addSanitizeItem: function () {
+  addSanitizeItem: function() {
     if (typeof Sanitizer != 'function')
       return;
     // Sanitizer will execute this
     Sanitizer.prototype.items['extensions-tabmix'] = {
-      clear : function() {
+      clear: function() {
         try {
           let win = Tabmix.getTopWin();
           win.Tabmix.Sanitizer.sanitize();
         } catch (ex) {
-          try { Components.utils.reportError(ex); } catch(e) {}
+          try {
+            Tabmix.reportError(ex);
+          } catch (e) { }
         }
       },
       get canClear() {
@@ -34,7 +41,7 @@ Tabmix.setSanitizer = {
     };
   },
 
-  addMenuItem: function () {
+  addMenuItem: function() {
     var prefs = document.getElementsByTagName("preferences")[0];
     var _item;
     var itemList = document.getElementById("itemList");
@@ -63,7 +70,7 @@ Tabmix.setSanitizer = {
       check.setAttribute("label", this._label);
       check.setAttribute("accesskey", this._accesskey);
       check.setAttribute("preference", prefName);
-      check.setAttribute("oncommand", "Tabmix.setSanitizer.confirm(this);");
+      Tabmix.setItem(check, "oncommand", "Tabmix.setSanitizer.confirm(this);");
       if (Services.prefs.prefHasUserValue(prefName)) {
         this.checked = Services.prefs.getBoolPref(prefName);
         check.setAttribute("checked", this.checked);
@@ -72,8 +79,7 @@ Tabmix.setSanitizer = {
         check.setAttribute("type", "checkbox");
         check.setAttribute("noduration", "true");
         itemList.setAttribute("rows", parseInt(itemList.getAttribute("rows")) + 1);
-      }
-      else if (container.childNodes.length > 1) {
+      } else if (container.childNodes.length > 1) {
         // don't add our checkbox to a row that already have 2 items
         let row = document.createElement("row");
         container.parentNode.appendChild(row);
@@ -84,7 +90,7 @@ Tabmix.setSanitizer = {
       if (this.isPromptDialog) {
         Tabmix.setSanitizer.disableMenuItem();
         pref.setAttribute("readonly", "true");
-        check.setAttribute("onsyncfrompreference",
+        Tabmix.setItem(check, "onsyncfrompreference",
                            "Tabmix.setSanitizer.checked = this.checked; " +
                            "return gSanitizePromptDialog.onReadGeneric();");
       }
@@ -92,7 +98,7 @@ Tabmix.setSanitizer = {
   },
 
   checked: false,
-  disableMenuItem: function () {
+  disableMenuItem: function() {
     let disabled = gSanitizePromptDialog.selectedTimespan !== Sanitizer.TIMESPAN_EVERYTHING;
     let checkbox = document.getElementById("extensions-tabmix");
     checkbox.setAttribute("disabled", disabled);
@@ -103,7 +109,7 @@ Tabmix.setSanitizer = {
     }
   },
 
-  confirm: function (aCheckbox) {
+  confirm: function(aCheckbox) {
     if (!aCheckbox.checked)
       return;
 
