@@ -1,24 +1,30 @@
-/* globals _sminstalled, gPreferenceList */
+/* exported  defaultSetting, toggleSyncPreference, exportData, importData,
+             showPane, openHelp */
+/* import-globals-from ../utils.js */
+/* import-globals-from shortcuts.js */
+/* import-globals-from menu.js */
+/* import-globals-from appearance.js */
 "use strict";
 
 /***** Preference Dialog Functions *****/
-const {classes: Cc, interfaces: Ci} = Components; // jshint ignore:line
-const PrefFn = {0: "", 32: "CharPref", 64: "IntPref", 128: "BoolPref"};
+var gIncompatiblePane;
+var {classes: Cc, interfaces: Ci, utils: Cu} = Components;
+var PrefFn = {0: "", 32: "CharPref", 64: "IntPref", 128: "BoolPref"};
 
-function $(id) document.getElementById(id)
+this.$ = id => document.getElementById(id);
 
-var gPrefWindow = { // jshint ignore:line
+var gPrefWindow = {
   widthChanged: false,
   _initialized: false,
   init: function() {
     this._initialized = true;
 
+    var prefWindow = $("TabMIxPreferences");
     /* Chromifox theme force button height to 25px */
     var skin = Services.prefs.getCharPref("general.skins.selectedSkin");
     if (skin == "cfxec")
       prefWindow.setAttribute("chromifox", true);
 
-    var prefWindow = $("TabMIxPreferences");
     if (TabmixSvc.isMac)
       prefWindow.setAttribute("mac", true);
     else if (TabmixSvc.isLinux) {
@@ -46,7 +52,7 @@ var gPrefWindow = { // jshint ignore:line
 
     // init buttons extra1, extra2, accept, cancel
     docElt.getButton("extra1").setAttribute("icon", "apply");
-    docElt.getButton("extra2").setAttribute("popup","tm-settings");
+    docElt.getButton("extra2").setAttribute("popup", "tm-settings");
     docElt.setAttribute("cancelbuttonlabel", docElt.mStrBundle.GetStringFromName("button-cancel"));
     this.setButtons(true);
 
@@ -70,9 +76,10 @@ var gPrefWindow = { // jshint ignore:line
     let style = window.getComputedStyle(content, "");
     let contentWidth = parseInt(style.width) + parseInt(style.marginRight) +
                        parseInt(style.marginLeft);
-    Array.slice(aPaneElement.getElementsByTagName("tabbox")).forEach(function(tabbox) {
+    let tabboxes = aPaneElement.getElementsByTagName("tabbox");
+    for (let tabbox of tabboxes) {
       diff = Math.max(diff, tabbox.boxObject.width - contentWidth);
-    });
+    }
     window.innerWidth += diff;
   },
 
@@ -86,22 +93,22 @@ var gPrefWindow = { // jshint ignore:line
 
   handleEvent: function(aEvent) {
     switch (aEvent.type) {
-    case "change":
-      if (aEvent.target.localName != "preference")
-        return;
-      this.updateBroadcaster(aEvent.target);
-      if (!this.instantApply)
-        this.updateApplyButton(aEvent);
-      break;
-    case "beforeaccept":
-      if (this.widthChanged)
-        gAppearancePane.changeTabsWidth();
-      if (!this.instantApply) {
-        // prevent TMP_SessionStore.setService from runing
-        Tabmix.getTopWin().tabmix_setSession = true;
-        Shortcuts.prefsChangedByTabmix = true;
-      }
-      break;
+      case "change":
+        if (aEvent.target.localName != "preference")
+          return;
+        this.updateBroadcaster(aEvent.target);
+        if (!this.instantApply)
+          this.updateApplyButton(aEvent);
+        break;
+      case "beforeaccept":
+        if (this.widthChanged)
+          gAppearancePane.changeTabsWidth();
+        if (!this.instantApply) {
+          // prevent TMP_SessionStore.setService from running
+          Tabmix.getTopWin().tabmix_setSession = true;
+          Shortcuts.prefsChangedByTabmix = true;
+        }
+        break;
     }
   },
 
@@ -183,11 +190,11 @@ var gPrefWindow = { // jshint ignore:line
     var broadcasters = $(paneID + ":Broadcaster");
     if (!broadcasters)
       return;
-    Array.forEach(broadcasters.childNodes, function (broadcaster) {
+    for (let broadcaster of broadcasters.childNodes) {
       let preference = $(broadcaster.id.replace("obs", "pref"));
       if (preference)
         this.updateBroadcaster(preference, broadcaster);
-    }, this);
+    }
   },
 
   updateBroadcaster: function(aPreference, aBroadcaster) {
@@ -203,13 +210,13 @@ var gPrefWindow = { // jshint ignore:line
   },
 
   setDisabled: function(itemOrId, val) {
-    var item = typeof(itemOrId) == "string" ? $(itemOrId) : itemOrId;
+    var item = typeof (itemOrId) == "string" ? $(itemOrId) : itemOrId;
     if (!item)
       return;
     if (item.hasAttribute("inverseDependency"))
       val = !val;
     // remove disabled when the value is false
-    Tabmix.setItem(item, "disabled" , val || null);
+    Tabmix.setItem(item, "disabled", val || null);
   },
 
   tabSelectionChanged: function(event) {
@@ -221,9 +228,14 @@ var gPrefWindow = { // jshint ignore:line
       tabs._inited = true;
       if (preference.value !== null)
         tabs.selectedIndex = preference.value;
-    }
-    else if (preference.value != tabs.selectedIndex)
+      else {
+        let val = preference.valueFromPreferences;
+        if (val !== null)
+          tabs.selectedIndex = val;
+      }
+    } else if (preference.value != tabs.selectedIndex) {
       preference.valueFromPreferences = tabs.selectedIndex;
+    }
   },
 
   afterShortcutsChanged: function() {
@@ -255,8 +267,8 @@ function getPrefByType(prefName) {
     var fn = PrefFn[Services.prefs.getPrefType(prefName)];
     if (fn == "CharPref")
       return Services.prefs.getComplexValue(prefName, Ci.nsISupportsString).data;
-    else
-      return Services.prefs["get" + fn](prefName);
+
+    return Services.prefs["get" + fn](prefName);
   } catch (ex) {
     Tabmix.log("can't read preference " + prefName + "\n" + ex, true);
   }
@@ -281,28 +293,28 @@ function setPref(aPref) {
     let str = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
     str.data = aPref.value;
     Services.prefs.setComplexValue(aPref.name, Ci.nsISupportsString, str);
-  }
-  else
+  } else {
     Services.prefs["set" + fn](aPref.name, aPref.value);
+  }
 }
 
 function setPrefAfterImport(aPref) {
   // in prev version we use " " for to export string to file
   aPref.value = aPref.value.replace(/^"*|"*$/g, "");
 
-  // preference that exist in the defaulbranch but no longer in use by Tabmix
+  // preference that exist in the default branch but no longer in use by Tabmix
   switch (aPref.name) {
-  case "browser.tabs.autoHide":
-    // from tabmix 0.3.6.0.080223 we use extensions.tabmix.hideTabbar
-    Tabmix.prefs.setIntPref("hideTabbar", aPref.value ? 1 : 0);
-    return true;
-  case "browser.tabs.closeButtons":
-    // we use browser.tabs.closeButtons only in 0.3.8.3
-    if (aPref.value < 0 || aPref.value > 6)
-      aPref.value = 6;
-    aPref.value = [3,5,1,1,2,4,1][aPref.value];
-    Tabmix.prefs.setIntPref("tabs.closeButtons", aPref.value);
-    return true;
+    case "browser.tabs.autoHide":
+      // from tabmix 0.3.6.0.080223 we use extensions.tabmix.hideTabbar
+      Tabmix.prefs.setIntPref("hideTabbar", aPref.value ? 1 : 0);
+      return true;
+    case "browser.tabs.closeButtons":
+      // we use browser.tabs.closeButtons only in 0.3.8.3
+      if (aPref.value < 0 || aPref.value > 6)
+        aPref.value = 6;
+      aPref.value = [3, 5, 1, 1, 2, 4, 1][aPref.value];
+      Tabmix.prefs.setIntPref("tabs.closeButtons", aPref.value);
+      return true;
   }
 
   // don't do anything if user locked a preference
@@ -331,35 +343,34 @@ function setPrefAfterImport(aPref) {
   return false;
 }
 
-let sessionPrefs = ["browser.sessionstore.resume_from_crash",
+var sessionPrefs = ["browser.sessionstore.resume_from_crash",
                     "browser.startup.page",
                     "extensions.tabmix.sessions.manager",
                     "extensions.tabmix.sessions.crashRecovery"];
 
 XPCOMUtils.defineLazyGetter(window, "gPreferenceList", function() {
   // other settings not in extensions.tabmix. branch that we save
-  let otherPrefs = ["browser.allTabs.previews","browser.ctrlTab.previews",
-  "browser.link.open_newwindow","browser.link.open_newwindow.override.external",
-  "browser.link.open_newwindow.restriction","browser.newtab.url",
-  "browser.search.context.loadInBackground","browser.search.openintab",
-  "browser.sessionstore.interval","browser.sessionstore.max_tabs_undo",
-  "browser.sessionstore.postdata","browser.sessionstore.privacy_level",
-  "browser.sessionstore.restore_on_demand",
-  "browser.sessionstore.resume_from_crash","browser.startup.page",
-  "browser.tabs.animate","browser.tabs.closeWindowWithLastTab",
-  "browser.tabs.insertRelatedAfterCurrent","browser.tabs.loadBookmarksInBackground",
-  "browser.tabs.loadDivertedInBackground","browser.tabs.loadInBackground",
-  "browser.tabs.tabClipWidth","browser.tabs.tabMaxWidth","browser.tabs.tabMinWidth",
-  "browser.tabs.warnOnClose","browser.warnOnQuit",
-  "toolkit.scrollbox.clickToScroll.scrollDelay","toolkit.scrollbox.smoothScroll"];
-
-  if (!Tabmix.isVersion(200))
-    otherPrefs.push("browser.warnOnRestart");
+  let otherPrefs = [
+    "browser.allTabs.previews", "browser.ctrlTab.previews",
+    "browser.link.open_newwindow", "browser.link.open_newwindow.override.external",
+    "browser.link.open_newwindow.restriction", TabmixSvc.newtabUrl,
+    "browser.search.context.loadInBackground", "browser.search.openintab",
+    "browser.sessionstore.interval", "browser.sessionstore.max_tabs_undo",
+    "browser.sessionstore.postdata", "browser.sessionstore.privacy_level",
+    "browser.sessionstore.restore_on_demand",
+    "browser.sessionstore.resume_from_crash", "browser.startup.page",
+    "browser.tabs.animate", "browser.tabs.closeWindowWithLastTab",
+    "browser.tabs.insertRelatedAfterCurrent", "browser.tabs.loadBookmarksInBackground",
+    "browser.tabs.loadDivertedInBackground", "browser.tabs.loadInBackground",
+    "browser.tabs.tabClipWidth", "browser.tabs.tabMaxWidth", "browser.tabs.tabMinWidth",
+    "browser.tabs.warnOnClose", "browser.warnOnQuit",
+    "toolkit.scrollbox.clickToScroll.scrollDelay", "toolkit.scrollbox.smoothScroll"
+  ];
 
   let prefs = Services.prefs.getDefaultBranch("");
   let tabmixPrefs = Services.prefs.getChildList("extensions.tabmix.").sort();
   // filter out preference without default value
-  tabmixPrefs = otherPrefs.concat(tabmixPrefs).filter(function(pref){
+  tabmixPrefs = otherPrefs.concat(tabmixPrefs).filter(function(pref) {
     try {
       return prefs["get" + PrefFn[prefs.getPrefType(pref)]](pref) !== undefined;
     } catch (ex) { }
@@ -368,7 +379,7 @@ XPCOMUtils.defineLazyGetter(window, "gPreferenceList", function() {
   return tabmixPrefs;
 });
 
-__defineGetter__("_sminstalled", function() {
+XPCOMUtils.defineLazyGetter(this, "_sminstalled", function() {
   return Tabmix.getTopWin().Tabmix.extensions.sessionManager;
 });
 
@@ -379,7 +390,7 @@ function defaultSetting() {
   Shortcuts.prefsChangedByTabmix = true;
   let SMinstalled = _sminstalled;
   let prefs = !SMinstalled ? gPreferenceList :
-      gPreferenceList.map(function(pref) sessionPrefs.indexOf(pref) == -1);
+      gPreferenceList.map(pref => sessionPrefs.indexOf(pref) == -1);
   prefs.forEach(function(pref) {
     Services.prefs.clearUserPref(pref);
   });
@@ -396,9 +407,9 @@ function toggleSyncPreference() {
   const sync = "services.sync.prefs.sync.";
   let fn = Tabmix.prefs.getBoolPref("syncPrefs") ? "clearUserPref" : "setBoolPref";
   Tabmix.prefs[fn]("syncPrefs", true);
-  let exclode = ["extensions.tabmix.sessions.onStart.sessionpath"];
+  let exclude = ["extensions.tabmix.sessions.onStart.sessionpath"];
   gPreferenceList.forEach(function(pref) {
-    if (exclode.indexOf(pref) == -1)
+    if (exclude.indexOf(pref) == -1)
       Services.prefs[fn](sync + pref, true);
   });
   Services.prefs.savePrefFile(null);
@@ -407,55 +418,42 @@ function toggleSyncPreference() {
 function exportData() {
   // save all pending changes
   gPrefWindow.onApply();
-
-  let patterns = gPreferenceList.map(function(pref) {
-    return pref + "=" + getPrefByType(pref) + "\n";
-  });
-  patterns[patterns.length-1] = patterns[patterns.length-1].replace(/\n$/, "");
-  patterns.unshift("tabmixplus\n");
-
-  function exportCallback(aFile) {
-    if (!/\.txt$/.test(aFile.leafName.toLowerCase()))
-      aFile.leafName += ".txt";
-    if (aFile.exists())
-      aFile.remove(true);
-    aFile.create(aFile.NORMAL_FILE_TYPE, parseInt("0666", 8));
-    let stream = Cc["@mozilla.org/network/file-output-stream;1"].
-                  createInstance(Ci.nsIFileOutputStream);
-    stream.init(aFile, 0x02, 0x200, null);
-    for (let i = 0; i < patterns.length ; i++)
-      stream.write(patterns[i], patterns[i].length);
-    stream.close();
-  }
-  showFilePicker("save", exportCallback);
+  showFilePicker("save").then(file => {
+    if (file) {
+      let patterns = gPreferenceList.map(function(pref) {
+        return "\n" + pref + "=" + getPrefByType(pref);
+      });
+      patterns.unshift("tabmixplus");
+      OS.File.writeAtomic(file.path, patterns.join(""), {
+        encoding: "utf-8", tmpPath: file.path + ".tmp"
+      });
+    }
+  }).then(null, Tabmix.reportError);
 }
 
-function importData () {
-  function importCallback(aFile) {
-    let stream = Cc["@mozilla.org/network/file-input-stream;1"].
-                createInstance(Ci.nsIFileInputStream);
-    stream.init(aFile, 0x01, parseInt("0444", 8), null);
-    let streamIO = Cc["@mozilla.org/scriptableinputstream;1"].
-                createInstance(Ci.nsIScriptableInputStream);
-    streamIO.init(stream);
-    let input = streamIO.read(stream.available());
-    streamIO.close();
-    stream.close();
-    if (input)
+function importData() {
+  showFilePicker("open").then(file => {
+    return file && OS.File.read(file.path);
+  }).then(input => {
+    if (input) {
+      let decoder = new TextDecoder();
+      input = decoder.decode(input);
       loadData(input.replace(/\r\n/g, "\n").split("\n"));
-  }
-
-  showFilePicker("open", importCallback);
+    }
+  }).then(null, Tabmix.reportError);
 }
 
-function showFilePicker(mode, callback) {
+/**
+ * Open file picker in open or save mode
+ *
+ * @param mode
+ *        The mode for the file picker: open|save
+ *
+ * @return Promise<{nsILocalFile}>
+ */
+function showFilePicker(mode) {
   const nsIFilePicker = Ci.nsIFilePicker;
   var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
-  function fpCallback(aResult) {
-    if (aResult != nsIFilePicker.returnCancel)
-      callback(fp.file);
-  }
-
   if (mode == "open")
     mode = nsIFilePicker.modeOpen;
   else {
@@ -465,16 +463,17 @@ function showFilePicker(mode, callback) {
   }
   fp.init(window, null, mode);
   fp.appendFilters(nsIFilePicker.filterText);
-  if (Tabmix.isVersion(180))
-    fp.open(fpCallback);
-  else
-    fpCallback(fp.show());
+  return AsyncUtils.spawnFn(fp, fp.open).then(aResult => {
+    return aResult != nsIFilePicker.returnCancel ? fp.file : null;
+  });
 }
 
-function loadData (pattern) {
-  if (pattern[0]!="tabmixplus") {
+function loadData(pattern) {
+  if (pattern[0] != "tabmixplus") {
     //  Can not import because it is not a valid file.
-    alert(TabmixSvc.getString("tmp.importPref.error1"));
+    let msg = TabmixSvc.getString("tmp.importPref.error1");
+    let title = TabmixSvc.getString("tabmixoption.error.title");
+    Services.prompt.alert(window, title, msg);
     return;
   }
 
@@ -500,14 +499,14 @@ function loadData (pattern) {
 
   var prefName, prefValue;
   Shortcuts.prefsChangedByTabmix = true;
-  for (let i = 1; i < pattern.length; i++){
-    let index = pattern[i].indexOf("=");
-    if (index > 0){
-      prefName  = pattern[i].substring(0,index);
-      if (SMinstalled && sessionPrefs.indexOf(prefName) > -1)
-        continue;
-      prefValue = pattern[i].substring(index+1,pattern[i].length);
-      setPrefByType(prefName, prefValue, true);
+  for (let i = 1; i < pattern.length; i++) {
+    let valIndex = pattern[i].indexOf("=");
+    if (valIndex > 0) {
+      prefName = pattern[i].substring(0, valIndex);
+      if (!SMinstalled || sessionPrefs.indexOf(prefName) == -1) {
+        prefValue = pattern[i].substring(valIndex + 1, pattern[i].length);
+        setPrefByType(prefName, prefValue, true);
+      }
     }
   }
   gPrefWindow.afterShortcutsChanged();
@@ -521,13 +520,13 @@ function loadData (pattern) {
 function showPane(paneID) {
   let docElt = document.documentElement;
   let paneToLoad = document.getElementById(paneID);
-  if(!paneToLoad || paneToLoad.nodeName != "prefpane")
+  if (!paneToLoad || paneToLoad.nodeName != "prefpane")
     paneToLoad = $(docElt.lastSelected);
   docElt.showPane(paneToLoad);
 }
 
 function openHelp(helpTopic) {
-  var helpPage = "http://tmp.garyr.net/help/#";
+  var helpPage = "http://tabmixplus.org/support/viewpage.php?t=3&p=";
   // Check if the help page already open in the top window
   var recentWindow = Tabmix.getTopWin();
   var tabBrowser = recentWindow.gBrowser;
@@ -537,6 +536,7 @@ function openHelp(helpTopic) {
       let browser = browsers[i];
       if (browser.currentURI.spec.startsWith(helpPage)) {
         tabBrowser.tabContainer.selectedIndex = i;
+        browser.tabmix_allowLoad = true;
         return true;
       }
     }
@@ -548,17 +548,18 @@ function openHelp(helpTopic) {
   if (!helpTopic) {
     var currentPane = document.documentElement.currentPane;
     helpTopic = currentPane.helpTopic;
-    if (currentPane.id == "paneSession" && helpTopic == "tabmix")
+    if (currentPane.id == "paneSession") {
       helpTopic = $("session").parentNode.selectedTab.getAttribute("helpTopic");
+    }
   }
-
+  helpTopic = helpTopic.toLowerCase().replace("mouse_-_", "").replace(/_-_|_/g, "-");
   recentWindow.openUILinkIn(helpPage + helpTopic, where);
 }
 
-var gIncompatiblePane = {
+gIncompatiblePane = {
   lastSelected: "paneLinks",
 
-  init: function (docElt) {
+  init: function(docElt) {
     this.paneButton = document.getAnonymousElementByAttribute(docElt, "pane", "paneIncompatible");
     let radioGroup = this.paneButton.parentNode;
     radioGroup.addEventListener("command", this, false);
@@ -570,7 +571,7 @@ var gIncompatiblePane = {
     radioGroup.removeEventListener("command", this, false);
   },
 
-  handleEvent: function (aEvent) {
+  handleEvent: function(aEvent) {
     if (aEvent.type != "command")
       return;
     let prefWindow = document.documentElement;
@@ -578,14 +579,14 @@ var gIncompatiblePane = {
       this.lastSelected = prefWindow.lastSelected;
   },
 
-  checkForIncompatible: function (aShowList) {
-     let tmp = { };
-     Components.utils.import("resource://tabmixplus/extensions/CompatibilityCheck.jsm", tmp);
-     tmp = new tmp.CompatibilityCheck(window, aShowList, true);
+  checkForIncompatible: function(aShowList) {
+    let tmp = {};
+    Components.utils.import("resource://tabmixplus/extensions/CompatibilityCheck.jsm", tmp);
+    tmp = new tmp.CompatibilityCheck(window, aShowList, true);
   },
 
   // call back function from CompatibilityCheck.jsm
-  hide_IncompatibleNotice: function (aHide, aFocus) {
+  hide_IncompatibleNotice: function(aHide, aFocus) {
     if (this.paneButton.collapsed != aHide) {
       this.paneButton.collapsed = aHide;
       $("paneIncompatible").collapsed = aHide;
@@ -607,4 +608,11 @@ XPCOMUtils.defineLazyGetter(gPrefWindow, "pinTabLabel", function() {
          win.document.getElementById("context_unpinTab").getAttribute("label");
 });
 
+XPCOMUtils.defineLazyGetter(this, "OS", () => {
+  return Cu.import("resource://gre/modules/osfile.jsm", {}).OS;
+});
+
+XPCOMUtils.defineLazyModuleGetter(this, "AsyncUtils",
+                                  "resource://tabmixplus/AsyncUtils.jsm");
+
 Tabmix.lazy_import(window, "Shortcuts", "Shortcuts", "Shortcuts");
