@@ -1,20 +1,19 @@
+/* globals TabmixSessionManager */
+/* exported gSessionPane */
 "use strict";
 
 var gSessionPane = {
   gSessionManager: null,
-  init: function () {
+  init: function() {
     if (TabmixSvc.isLinux)
       $("sessionManager-panels").setAttribute("linux", "true");
 
-    this.setVisiblecontent(!!this.sessionManagerAddon, true);
+    this.setVisibleContent(Boolean(this.sessionManagerAddon), true);
 
     gPrefWindow.setDisabled("obs_ss_postdata", $("pref_ss_postdata").value == 2);
     this.isSessionStoreEnabled(true);
     this.updateSessionShortcuts();
 
-    if (Tabmix.isVersion(200))
-      gPrefWindow.removeChild("pref_browser.warnOnRestart");
-
     gPrefWindow.initPane("paneSession");
   },
 
@@ -23,24 +22,24 @@ var gSessionPane = {
       gMenuPane.updateSessionShortcuts();
   },
 
-  isSessionStoreEnabled: function (onStart) {
+  isSessionStoreEnabled: function(onStart) {
     if (this.gSessionManager)
       return;
 
     var sessionStoreEnabled = Services.prefs.getIntPref("browser.startup.page") == 3 ||
         Services.prefs.getBoolPref("browser.sessionstore.resume_from_crash");
     $("sessionsOptions").checked = sessionStoreEnabled;
-    $("sesionsPanel").setAttribute("manager", !sessionStoreEnabled ? "tabmix" : "firefox");
+    $("sessionsPanel").setAttribute("manager", !sessionStoreEnabled ? "tabmix" : "firefox");
     if (!onStart || sessionStoreEnabled)
       $("session").selectedIndex = sessionStoreEnabled ? 2 : 0;
     else if ($("session").selectedIndex == 2)
       $("session").selectedIndex = 0;
   },
 
-  setSessionsOptions: function (item) {
+  setSessionsOptions: function(item) {
     let instantApply = document.documentElement.instantApply;
     var useSessionManager = !item.checked;
-    $("sesionsPanel").setAttribute("manager", useSessionManager ? "tabmix" : "firefox");
+    $("sessionsPanel").setAttribute("manager", useSessionManager ? "tabmix" : "firefox");
 
     function updatePrefs(aItemId, aValue) {
       let preference = $("pref_" + aItemId);
@@ -64,9 +63,6 @@ var gSessionPane = {
 
     // sessionstore pref
     function sessionstorePrefs() {
-      // browser.warnOnRestart remove on Firefox 20
-      if (!Tabmix.isVersion(200))
-        updatePrefs("browser.warnOnRestart", !useSessionManager);
       updatePrefs("browser.warnOnQuit", !useSessionManager);
       updatePrefs("resume_from_crash", !useSessionManager);
       // "browser.startup.page"
@@ -77,8 +73,7 @@ var gSessionPane = {
     if (useSessionManager) {
       sessionstorePrefs();
       sessionPrefs();
-    }
-    else {
+    } else {
       sessionPrefs();
       sessionstorePrefs();
     }
@@ -90,7 +85,7 @@ var gSessionPane = {
       gPrefWindow.setButtons(!gPrefWindow.changes.length);
   },
 
-  setSessionpath: function (val) {
+  setSessionpath: function(val) {
     var menuItem = $("onStart.popup").getElementsByAttribute("value", val)[0];
     $("pref_sessionpath").value = menuItem.getAttribute("session");
   },
@@ -106,7 +101,7 @@ var gSessionPane = {
     return sm && sm.gSessionManager || sm;
   },
 
-  setVisiblecontent: function (sessionManagerInstalled, onStart) {
+  setVisibleContent: function(sessionManagerInstalled, onStart) {
     if (typeof sessionManagerInstalled != "boolean")
       return;
 
@@ -118,21 +113,20 @@ var gSessionPane = {
       $("sessionmanager_button").setAttribute("image", "chrome://sessionmanager/skin/icon.png");
       if (onStart)
         $("chooseFile").selectedIndex = 1;
-    }
-    else {
+    } else {
       this.isSessionStoreEnabled(onStart);
       TabmixSessionManager.createMenuForDialog($("onStart.popup"));
       $("onStart.loadsession").value = $("pref_onStart.loadsession").value;
     }
   },
 
-  sessionManagerOptions: function () {
+  sessionManagerOptions: function() {
     this.sessionManagerAddon.openOptions();
   },
 
-  convertSession: function () {
+  convertSession: function() {
     var browserWindow = Tabmix.getTopWin();
-    if ($("chooseFile").selectedItem.value === 0)
+    if ($("chooseFile").selectedItem.value == "0")
       browserWindow.TabmixConvertSession.selectFile(window);
     else
       browserWindow.TabmixConvertSession.convertFile();
