@@ -2,15 +2,18 @@
  * based on keyconfig extension by dorando
  */
 
+/* exported getKeysForShortcut */
 "use strict";
 
+var getFormattedKey = key => Shortcuts.getFormattedKey(key);
+
 ///XXX TODO - add black list with shortcut (F1, Ctrl-Tab ....) that don't have DOM key element
 function getKeysForShortcut(shortcut, id, win) {
   if (!win)
     win = Services.wm.getMostRecentWindow("navigator:browser");
 
-  let $ = function(id) id && win.document.getElementById(id);
-  let isDisabled = function(item) item && item.getAttribute("disabled");
+  let $ = aID => aID && win.document.getElementById(aID);
+  let isDisabled = item => item && item.getAttribute("disabled");
 
   let ourKey = $(id);
   if (isDisabled(ourKey))
@@ -18,7 +21,7 @@ function getKeysForShortcut(shortcut, id, win) {
 
   let dots = "…";
   let keys = win.document.getElementsByTagName("key");
-  let usedKeys = Array.filter(keys, function(key) {
+  let usedKeys = Array.prototype.filter.call(keys, function(key) {
     if (ourKey == key)
       return false;
     if (isDisabled(key) || isDisabled($(key.getAttribute("command"))))
@@ -28,10 +31,8 @@ function getKeysForShortcut(shortcut, id, win) {
       key: key.getAttribute("key"),
       keycode: key.getAttribute("keycode")
     };
-    if (getFormattedKey(_key) == shortcut)
-      return true;
-    return false;
-  }).map(function(key) "     " + _getKeyName(win, key).replace(dots, ""));
+    return getFormattedKey(_key) == shortcut;
+  }).map(key => "     " + _getKeyName(win, key).replace(dots, ""));
 
   return usedKeys.join("\n");
 }
@@ -65,22 +66,26 @@ function _getKeyName(win, aKey) {
     return aKey.getAttribute("label");
 
   let id = command && command.indexOf(":") > -1 ? command :
-           aKey.id.replace(/xxx_key.+?_/, "") || // keyconfig foramt
+           aKey.id.replace(/xxx_key.+?_/, "") || // keyconfig format
            command || aKey.getAttribute("oncommand");
   let gUnicodeConverter = Cc['@mozilla.org/intl/scriptableunicodeconverter']
           .createInstance(Ci.nsIScriptableUnicodeConverter);
   gUnicodeConverter.charset = "UTF-8";
   try {
     id = gUnicodeConverter.ConvertToUnicode(id);
-  } catch(ex) { }
+  } catch (ex) { }
 
   let keyname = {
     action: Services.prefs.getIntPref("browser.backspace_action"),
     "BrowserReload();": "key_reload",
     goBackKb2: "goBackKb",
     goForwardKb2: "goForwardKb",
-    get cmd_handleBackspace() this.action < 2 && ["goBackKb", "cmd_scrollPageUp"][this.action],
-    get cmd_handleShiftBackspace() this.action < 2 && ["goForwardKb", "cmd_scrollPageDown"][this.action]
+    get cmd_handleBackspace() {
+      return this.action < 2 && ["goBackKb", "cmd_scrollPageUp"][this.action];
+    },
+    get cmd_handleShiftBackspace() {
+      return this.action < 2 && ["goForwardKb", "cmd_scrollPageDown"][this.action];
+    }
   };
   if (keyname[id]) {
     let key = doc.getElementById(keyname[id]);
@@ -104,7 +109,7 @@ function _getLabel(elm, attr, value) {
   return null;
 }
 
-function _getPath(elm){
+function _getPath(elm) {
   let names = [];
   while (elm && elm.localName && elm.localName != "toolbaritem" &&
          elm.localName != "popupset" && elm.id != "titlebar-content") {
@@ -114,5 +119,3 @@ function _getPath(elm){
   }
   return names.join(" > ");
 }
-
-function getFormattedKey(key) Shortcuts.getFormattedKey(key)
