@@ -1,10 +1,11 @@
+/* exported tabstyles */
 "use strict";
 
-function $(id) document.getElementById(id)
+var $ = id => document.getElementById(id);
 
-let tabstyles = { // jshint ignore:line
+var tabstyles = {
   pref: "appearance_tab",
-  init: function () {
+  init: function() {
     $("stylestabs").selectedIndex = Tabmix.prefs.prefHasUserValue(this.pref) ?
         Tabmix.prefs.getIntPref(this.pref) : 0;
 
@@ -21,30 +22,29 @@ let tabstyles = { // jshint ignore:line
     extra.classList.add("text-link");
   },
 
-  save: function () {
+  save: function() {
     Tabmix.prefs.setIntPref(this.pref, $("stylestabs").selectedIndex);
     // store the pref immediately
     Services.prefs.savePrefFile(null);
   },
 
-  cancel: function () {
-    Array.forEach($("stylespanels").childNodes, function(panel) {
+  cancel: function() {
+    let panels = $("stylespanels").childNodes;
+    for (let panel of panels) {
       $(panel.id)._ondialogcancel();
-    });
+    }
     this.save();
   },
 
-  openHelp: function () {
-    var subPage = ["Current_Tab", "Unloaded_tabs", "Unread_tabs", "Other_Tabs", "Progress_meter_on_tabs"];
-    var index = $("AppearanceTabBox").selectedIndex;
+  openHelp: function() {
     var win = window.opener || Tabmix.getTopWin();
     if (win)
-      win.openHelp("Customize_Styles_-_" + subPage[index]);
+      win.openHelp("display-tab#customize_styles");
     else
       document.documentElement.getButton("help").disabled = true;
   },
 
-  toggleRGBvisibility: function () {
+  toggleRGB_visibility: function() {
     let doc = document.documentElement;
     let extra = doc.getButton("extra2");
     let item = $("hide-RGB");
