@@ -1,6 +1,6 @@
 "use strict";
 
-var Tabmix = { // jshint ignore:line
+var Tabmix = {
   get prefs() {
     delete this.prefs;
     return (this.prefs = Services.prefs.getBranch("extensions.tabmix."));
@@ -23,19 +23,19 @@ var Tabmix = { // jshint ignore:line
 
   // Show/hide one item (specified via name or the item element itself).
   showItem: function(aItemOrId, aShow) {
-    var item = typeof(aItemOrId) == "string" ? document.getElementById(aItemOrId) : aItemOrId;
-    if (item && item.hidden == aShow)
+    var item = typeof (aItemOrId) == "string" ? document.getElementById(aItemOrId) : aItemOrId;
+    if (item && item.hidden == Boolean(aShow))
       item.hidden = !aShow;
   },
 
   setItem: function(aItemOrId, aAttr, aVal) {
-    var elem = typeof(aItemOrId) == "string" ? document.getElementById(aItemOrId) : aItemOrId;
+    var elem = typeof (aItemOrId) == "string" ? document.getElementById(aItemOrId) : aItemOrId;
     if (elem) {
       if (aVal === null || aVal === undefined) {
         elem.removeAttribute(aAttr);
         return;
       }
-      if (typeof(aVal) == "boolean")
+      if (typeof (aVal) == "boolean")
         aVal = aVal ? "true" : "false";
 
       if (!elem.hasAttribute(aAttr) || elem.getAttribute(aAttr) != aVal)
@@ -44,7 +44,7 @@ var Tabmix = { // jshint ignore:line
   },
 
   setAttributeList: function(aItemOrId, aAttr, aValue, aAdd) {
-    let elem = typeof(aItemOrId) == "string" ? document.getElementById(aItemOrId) : aItemOrId;
+    let elem = typeof (aItemOrId) == "string" ? document.getElementById(aItemOrId) : aItemOrId;
     let att = elem.getAttribute(aAttr);
     let array = att ? att.split(" ") : [];
     let index = array.indexOf(aValue);
@@ -75,7 +75,7 @@ var Tabmix = { // jshint ignore:line
     //   user are not in single window mode or
     //   there is no other window with the same privacy type
     return !this.getSingleWindowMode() ||
-      this.isVersion(200) && !this.RecentWindow.getMostRecentBrowserWindow({private: isPrivate});
+      !this.RecentWindow.getMostRecentBrowserWindow({private: isPrivate});
   },
 
   lazy_import: function(aObject, aName, aModule, aSymbol, aFlag, aArg) {
@@ -83,8 +83,8 @@ var Tabmix = { // jshint ignore:line
       this[aModule + "Initialized"] = false;
     var self = this;
     XPCOMUtils.defineLazyGetter(aObject, aName, function() {
-      let tmp = { };
-      Components.utils.import("resource://tabmixplus/"+aModule+".jsm", tmp);
+      let tmp = {};
+      Components.utils.import("resource://tabmixplus/" + aModule + ".jsm", tmp);
       let Obj = tmp[aSymbol];
       if ("prototype" in tmp[aSymbol])
         Obj = new Obj();
@@ -102,7 +102,7 @@ var Tabmix = { // jshint ignore:line
 
     var self = this;
     Object.defineProperty(aObject, aOldName, {
-      get: function () {
+      get: function() {
         self.informAboutChangeInTabmix(aOldName, aNewName);
         delete aObject[aOldName];
         return (aObject[aOldName] = self.getObject(window, aNewName));
@@ -122,12 +122,12 @@ var Tabmix = { // jshint ignore:line
       let extensionName = index > -1 ?
          path.charAt(2).toUpperCase() + path.substr(3, index) + " " : "";
       this.clog(err.message + "\n\n" + extensionName + "extension call " + aOldName +
-                 " from:\n" + "file: " + "chrome:" + path + "\nline: " + line +
+                 " from:\nfile: chrome:" + path + "\nline: " + line +
                  "\n\nPlease inform Tabmix Plus developer" +
-                 (extensionName ? ( " and " + extensionName + "developer.") : "."));
-    }
-    else
+                 (extensionName ? (" and " + extensionName + "developer.") : "."));
+    } else {
       this.clog(err.message + "\n\n" + stack);
+    }
   },
 
   promptService: function(intParam, strParam, aWindow, aCallBack) {
@@ -143,7 +143,7 @@ var Tabmix = { // jshint ignore:line
 
     // we use non modal dialog when we call for prompt on startup
     // when we don't have a callBack function use modal dialog
-    let modal = typeof(aCallBack) != "function";
+    let modal = typeof (aCallBack) != "function";
     var i;
     for (i = 0; i < intParam.length; i++)
       dpb.SetInt(i, intParam[i]);
@@ -152,17 +152,18 @@ var Tabmix = { // jshint ignore:line
     for (i = 0; i < strParam.length; i++)
       dpb.SetString(i, strParam[i]);
 
-    if (typeof(aWindow) == "undefined") {
-      try { aWindow = window;
-      }
-      catch (e) { aWindow = null;
+    if (typeof (aWindow) == "undefined") {
+      try {
+        aWindow = window;
+      } catch (e) {
+        aWindow = null;
       }
     }
 
     // we add dependent to features to make this dialog float over the window on start
     var dialog = Services.ww.openWindow(aWindow,
-           "chrome://tabmixplus/content/session/promptservice.xul","","centerscreen" +
-           (modal ? ",modal" : ",dependent") ,dpb);
+           "chrome://tabmixplus/content/session/promptservice.xul", "", "centerscreen" +
+           (modal ? ",modal" : ",dependent"), dpb);
     if (!modal)
       dialog._callBackFunction = aCallBack;
 
@@ -171,7 +172,7 @@ var Tabmix = { // jshint ignore:line
   },
 
   windowEnumerator: function Tabmix_windowEnumerator(aWindowtype) {
-    if (typeof(aWindowtype) == "undefined")
+    if (typeof (aWindowtype) == "undefined")
       aWindowtype = "navigator:browser";
     return Services.wm.getEnumerator(aWindowtype);
   },
@@ -181,11 +182,13 @@ var Tabmix = { // jshint ignore:line
     var count = 0;
     while (enumerator.hasMoreElements()) {
       let win = enumerator.getNext();
-      if ("TabmixSessionManager" in win && win.TabmixSessionManager.windowClosed)
-        continue;
-      count++;
-      if (!all && count == 2)
-        break;
+      let isClosed = "TabmixSessionManager" in win &&
+          win.TabmixSessionManager.windowClosed;
+      if (!isClosed) {
+        count++;
+        if (!all && count == 2)
+          break;
+      }
     }
     return count;
   },
@@ -202,7 +205,10 @@ var Tabmix = { // jshint ignore:line
     return window;
   },
 
-  compare: function TMP_utils_compare(a, b, lessThan) {return lessThan ? a < b : a > b;},
+  compare: function TMP_utils_compare(a, b, lessThan) {
+    return lessThan ? a < b : a > b;
+  },
+
   itemEnd: function TMP_utils_itemEnd(item, end) {
     return item.boxObject.screenX + (end ? item.getBoundingClientRect().width : 0);
   },
@@ -215,7 +221,7 @@ var Tabmix = { // jshint ignore:line
   // caller list
   _getMethod: function TMP_console_wrapper(id, args) {
     if (["changeCode", "setNewFunction", "nonStrictMode"].indexOf(id) > -1) {
-      this.installeChangecode();
+      this.installChangecode();
       return this[id].apply(this, args);
     }
     if (typeof TabmixSvc.console[id] == "function") {
@@ -225,20 +231,16 @@ var Tabmix = { // jshint ignore:line
     return null;
   },
 
-  installeChangecode: function() {
+  installChangecode: function() {
     Services.scriptloader.loadSubScript("chrome://tabmixplus/content/changecode.js", window);
-    this.installeChangecode = function() {};
+    this.installChangecode = function() {};
   },
 
   _init: function() {
     Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
     Components.utils.import("resource://gre/modules/Services.jsm");
-    this.lazy_import(window, "TabmixSvc", "Services", "TabmixSvc");
-    if (this.isVersion(200)) {
-      let resource = this.isVersion(210) ? "resource:///" : "resource://gre/";
-      XPCOMUtils.defineLazyModuleGetter(this, "RecentWindow",
-                 resource + "modules/RecentWindow.jsm");
-    }
+    XPCOMUtils.defineLazyModuleGetter(this, "RecentWindow",
+                                      "resource:///modules/RecentWindow.jsm");
 
     window.addEventListener("unload", function tabmix_destroy() {
       window.removeEventListener("unload", tabmix_destroy, false);
@@ -247,11 +249,12 @@ var Tabmix = { // jshint ignore:line
 
     var methods = ["changeCode", "setNewFunction", "nonStrictMode",
                    "getObject", "log", "getCallerNameByIndex", "callerName",
-                   "isCallerInList", "obj", "assert", "trace"];
+                   "clog", "isCallerInList", "callerTrace",
+                   "obj", "assert", "trace", "reportError"];
     methods.forEach(function(id) {
       this[id] = function TMP_console_wrapper() {
         return this._getMethod(id, arguments);
-      };
+      }.bind(this);
     }, this);
   },
 
@@ -264,3 +267,4 @@ var Tabmix = { // jshint ignore:line
 };
 
 Tabmix._init();
+Tabmix.lazy_import(window, "TabmixSvc", "Services", "TabmixSvc");
